@@ -1,46 +1,39 @@
 #/*##########################################################################
-# Copyright (C) 2004-2011 European Synchrotron Radiation Facility
+# Copyright (C) 2004 - 2012 European Synchrotron Radiation Facility
 #
-# This file is part of the PyMCA X-ray Fluorescence Toolkit developed at
-# the ESRF by the Beamline Instrumentation Software Support (BLISS) group.
+# This file is part of the PyMca X-ray Fluorescence Toolkit developed at
+# the ESRF by the Software group.
 #
-# This toolkit is free software; you can redistribute it and/or modify it 
-# under the terms of the GNU General Public License as published by the Free
-# Software Foundation; either version 2 of the License, or (at your option) 
+# This file is free software; you can redistribute it and/or modify it
+# under the terms of the GNU Lesser General Public License as published by the
+# Free Software Foundation; either version 2 of the License, or (at your option)
 # any later version.
 #
-# PyMCA is distributed in the hope that it will be useful, but WITHOUT ANY
+# This file is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 # details.
 #
-# You should have received a copy of the GNU General Public License along with
-# PyMCA; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
-# Suite 330, Boston, MA 02111-1307, USA.
-#
-# PyMCA follows the dual licensing model of Trolltech's Qt and Riverbank's PyQt
-# and cannot be used as a free plugin for a non-free program. 
-#
-# Please contact the ESRF industrial unit (industry@esrf.fr) if this license 
-# is a problem for you.
 #############################################################################*/
+__author__ = "E. Papillon & V.A. Sole - ESRF Software Group"
 import sys
-import string
 if sys.version < '3.0':
     import ConfigParser
 else:
     import configparser as ConfigParser
 try:
-    import numpy.oldnumeric as Numeric
+    import numpy
+    USE_NUMPY = True
 except:
-    pass
+    # do not use numpy, use lists
+    USE_NUMPY = False
 
 class ConfigDict(dict):
 
     def __init__(self, defaultdict={}, initdict=None, filelist=None):
         dict.__init__(self, defaultdict)
-        self.default= defaultdict
-        self.filelist= []
+        self.default = defaultdict
+        self.filelist = []
 
         if initdict is not None:
             self.update(initdict)
@@ -57,14 +50,15 @@ class ConfigDict(dict):
         """ Clear dictionnary
         """
         dict.clear(self)
-        self.filelist= []
+        self.filelist = []
 
     def _check(self):
         pass
 
     def __tolist(self, mylist):
-        if mylist is None: return None
-        if type(mylist)!=type([]):
+        if mylist is None:
+            return None
+        if type(mylist) != type([]):
             return [ mylist ]
         else:
             return mylist
@@ -73,7 +67,7 @@ class ConfigDict(dict):
         return self.filelist
 
     def getlastfile(self):
-        return self.filelist[len(self.filelist)-1]
+        return self.filelist[len(self.filelist) - 1]
 
     def __convert(obj, option):
         return option
@@ -82,33 +76,33 @@ class ConfigDict(dict):
         """
         read the input filename into the internal dictionary
         """
-        filelist= self.__tolist(filelist)
-        sections= self.__tolist(sections)
+        filelist = self.__tolist(filelist)
+        sections = self.__tolist(sections)
 
-        cfg= ConfigParser.ConfigParser()
-        cfg.optionxform= self.__convert
+        cfg = ConfigParser.ConfigParser()
+        cfg.optionxform = self.__convert
         cfg.read(filelist)
         self.__read(cfg, sections)
 
-        for file in filelist:
-                self.filelist.append([file, sections])
+        for ffile in filelist:
+            self.filelist.append([ffile, sections])
         self._check()
 
     def __read(self, cfg, sections=None):
-        cfgsect= cfg.sections()
+        cfgsect = cfg.sections()
 
         if sections is None:
-            readsect= cfgsect
+            readsect = cfgsect
         else:
-            readsect= [ sect for sect in cfgsect if sect in sections ]
+            readsect = [ sect for sect in cfgsect if sect in sections ]
 
         for sect in readsect:
-            ddict= self
+            ddict = self
             for subsectw in sect.split('.'):
                 subsect = subsectw.replace("_|_",".")
                 if not (subsect in ddict):
                     ddict[subsect]= {}
-                ddict= ddict[subsect]
+                ddict = ddict[subsect]
             for opt in cfg.options(sect):
                 ddict[opt]= self.__parse_data(cfg.get(sect, opt))
 
@@ -116,22 +110,22 @@ class ConfigDict(dict):
         if len(data):
             if data.find(',')== -1:
                 #it is not a list
-                if (data[0] == '[') and (data[-1] == ']'):
+                if USE_NUMPY and (data[0] == '[') and (data[-1] == ']'):
                     #this looks as an array
                     try:
-                        return Numeric.array([float(x) for x in data[1:-1].split()])
+                        return numpy.array([float(x) for x in data[1:-1].split()])
                     except:
                         try:
                             if (data[2] == '[') and (data[-3] == ']'):
                                 nrows = len(data[3:-3].split('] ['))
                                 indata = data[3:-3].replace('] [',' ')
-                                indata = Numeric.array([float(x) for x in \
+                                indata = numpy.array([float(x) for x in \
                                                            indata.split()])
                                 indata.shape = nrows,-1
                                 return indata
                         except:
                             pass
-        dataline= [ line for line in data.splitlines() ]
+        dataline = [ line for line in data.splitlines() ]
         if len(dataline)==1:
             return self.__parse_line(dataline[0])
         else:
@@ -139,7 +133,8 @@ class ConfigDict(dict):
         
     def __parse_line(self, line):
         if line.find(',')!=-1:
-            return [ self.__parse_string(sstr.strip()) for sstr in line.split(',') ]
+            return [ self.__parse_string(sstr.strip())\
+                     for sstr in line.split(',') ]
         else:
             return self.__parse_string(line.strip())
 
@@ -154,8 +149,8 @@ class ConfigDict(dict):
 
     def tostring(self, sections=None):
         import StringIO
-        tmp= StringIO.StringIO()
-        sections= self.__tolist(sections)
+        tmp = StringIO.StringIO()
+        sections = self.__tolist(sections)
         self.__write(tmp, self, sections)
         return tmp.getvalue()
 
@@ -163,13 +158,12 @@ class ConfigDict(dict):
         """
         Write the current dictionary to the given filename
         """
-        sections= self.__tolist(sections)
-        fp= open(filename, "w")
+        sections = self.__tolist(sections)
+        fp = open(filename, "w")
         self.__write(fp, self, sections)
         fp.close()
 
     def __write(self, fp, ddict, sections=None, secthead=None):
-        retstring= ''
         dictkey= []
         listkey= []
         valkey= []
@@ -182,13 +176,15 @@ class ConfigDict(dict):
                 valkey.append(key)
 
         for key in valkey:
-            if type(ddict[key])== Numeric.ArrayType:
-                fp.write('%s =' % key + ' [ '+ ' '.join([str(val) for val in ddict[key]])+' ]\n')
-            else:
-                fp.write('%s = %s\n'%(key, ddict[key]))
+            if USE_NUMPY:
+                if type(ddict[key])== numpy.ndarray:
+                    fp.write('%s =' % key + ' [ '+\
+                             ' '.join([str(val) for val in ddict[key]])+' ]\n')
+                    continue
+            fp.write('%s = %s\n'% (key, ddict[key]))
 
         for key in listkey:
-            fp.write('%s = '%key)
+            fp.write('%s = ' % key)
             llist= []
             sep= ', '
             for item in ddict[key]:
@@ -204,12 +200,12 @@ class ConfigDict(dict):
         #    print "dictkey after = ",dictkey
         for key in dictkey:
             if secthead is None:
-                newsecthead= key.replace(".","_|_")
+                newsecthead= key.replace(".", "_|_")
             else:
-                newsecthead = '%s.%s'%(secthead, key.replace(".","_|_"))
-            #print "newsecthead = ",newsecthead
-            fp.write('\n[%s]\n'%newsecthead)
-            self.__write(fp, ddict[key], key,newsecthead)
+                newsecthead = '%s.%s' % (secthead, key.replace(".","_|_"))
+            #print "newsecthead = ", newsecthead
+            fp.write('\n[%s]\n' % newsecthead)
+            self.__write(fp, ddict[key], key, newsecthead)
 
 
 def prtdict(ddict, lvl=0):
@@ -225,7 +221,6 @@ def prtdict(ddict, lvl=0):
                 print('-', key, '=', ddict[key])
 
 def test():
-    import sys
     if len(sys.argv)>1:
         config= ConfigDict(filelist=sys.argv[1:])
         prtdict(config)
