@@ -1,36 +1,30 @@
 #/*##########################################################################
-# Copyright (C) 2004-2011 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
 #
-# This file is part of the PyMCA X-ray Fluorescence Toolkit developed at
-# the ESRF by the Beamline Instrumentation Software Support (BLISS) group.
+# This file is part of the PyMca X-ray Fluorescence Toolkit developed at
+# the ESRF by the Software group.
 #
-# This toolkit is free software; you can redistribute it and/or modify it 
-# under the terms of the GNU General Public License as published by the Free
-# Software Foundation; either version 2 of the License, or (at your option) 
+# This file is free software; you can redistribute it and/or modify it
+# under the terms of the GNU Lesser General Public License as published by the
+# Free Software Foundation; either version 2 of the License, or (at your option)
 # any later version.
 #
-# PyMCA is distributed in the hope that it will be useful, but WITHOUT ANY
+# This file is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 # details.
 #
-# You should have received a copy of the GNU General Public License along with
-# PyMCA; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
-# Suite 330, Boston, MA 02111-1307, USA.
-#
-# PyMCA follows the dual licensing model of Trolltech's Qt and Riverbank's PyQt
-# and cannot be used as a free plugin for a non-free program. 
-#
-# Please contact the ESRF industrial unit (industry@esrf.fr) if this license 
-# is a problem for you.
 #############################################################################*/
-import DataObject
-import EdfFile
-import EdfFileDataSource
-import numpy.oldnumeric as Numeric
+__author__ = "V.A. Sole - ESRF Data Analysis"
+from PyMca import DataObject
+from PyMca import EdfFile
+from PyMca import EdfFileDataSource
 import numpy
 import sys
 import os
+
+# Offer automatic conversion to HDF5 in case of lacking
+# memory to hold the Stack.
 HDF5 = False
 try:
     import h5py
@@ -172,7 +166,7 @@ class EDFStack(DataObject.DataObject):
                                             "HDF5 files *.h5")
                                 if not len(hdf5file):
                                     raise IOError("Invalid output file")
-                                hdf5file = str(hdf5file)
+                                hdf5file = qt.safe_str(hdf5file)
                                 if not hdf5file.endswith(".h5"):
                                     hdf5file += ".h5"
                                 hdf, self.data =  ArraySave.getHDF5FileInstanceAndBuffer(hdf5file,
@@ -364,7 +358,7 @@ class EDFStack(DataObject.DataObject):
                                         if not len(hdf5file):
                                             raise IOError(\
                                                 "Invalid output file")
-                                        hdf5file = str(hdf5file)
+                                        hdf5file = qt.safe_str(hdf5file)
                                         if not hdf5file.endswith(".h5"):
                                             hdf5file += ".h5"
                                         hdf, self.data =  ArraySave.getHDF5FileInstanceAndBuffer(hdf5file,
@@ -383,9 +377,27 @@ class EDFStack(DataObject.DataObject):
                             self.incrProgressBar += 1
                             self.onProgress(self.incrProgressBar)
                     else:
+                        # test for ID24 map
+                        ID24 = False
+                        if "_sample_" in filelist[0]:
+                            i0StartFile = filelist[0].replace("_sample_", "_I0start_")
+                            if os.path.exists(i0StartFile):
+                                ID24 = True
+                                id24idx = 0
+                                i0Start = EdfFile.EdfFile(i0StartFile, 'rb').GetData(0).astype(numpy.float)
+                                i0EndFile = filelist[0].replace("_sample_", "_I0end_")
+                                i0Slope = 0.0
+                                if os.path.exists(i0EndFile):
+                                    i0End = EdfFile.EdfFile(i0EndFile, 'rb').GetData(0)
+                                    i0Slope = (i0End-i0Start)/len(filelist)
                         for tempEdfFileName in filelist:
                             tempEdf=EdfFile.EdfFile(tempEdfFileName, 'rb')
-                            pieceOfStack=tempEdf.GetData(0)
+                            if ID24:
+                                pieceOfStack=-numpy.log(tempEdf.GetData(0)/(i0Start[0,:] + id24idx * i0Slope))
+                                pieceOfStack[numpy.isfinite(pieceOfStack) == False] = 1
+                                id24idx += 1
+                            else:
+                                pieceOfStack=tempEdf.GetData(0)
                             self.data[self.incrProgressBar, :,:] = pieceOfStack[:,:]
                             self.incrProgressBar += 1
                             self.onProgress(self.incrProgressBar)
@@ -434,8 +446,6 @@ class EDFStack(DataObject.DataObject):
         name = os.path.basename(filename)
         n = len(name)
         i = 1
-        numbers = ['0', '1', '2', '3', '4', '5',
-                   '6', '7', '8','9']
         while (i <= n):
             c = name[n-i:n-i+1]
             if c in ['0', '1', '2',
@@ -553,10 +563,10 @@ if __name__ == "__main__":
         import QtBlissGraph
         w = QtBlissGraph.QtBlissGraph()
         graph = w
-    print("shape sum 0 = ",Numeric.sum(stack.data, 0).shape)
-    print("shape sum 1 = ",Numeric.sum(stack.data, 1).shape)
-    print("shape sum 2 = ",Numeric.sum(stack.data, 2).shape)
-    a = Numeric.sum(stack.data, imax)
+    print("shape sum 0 = ",numpy.sum(stack.data, 0).shape)
+    print("shape sum 1 = ",numpy.sum(stack.data, 1).shape)
+    print("shape sum 2 = ",numpy.sum(stack.data, 2).shape)
+    a = numpy.sum(stack.data, imax)
     print(a.shape)
     graph.setX1AxisLimits(0, a.shape[0])
     if 0:
@@ -574,9 +584,9 @@ if __name__ == "__main__":
     w.show()
 
     if imax == 0:
-        mcaData0 = Numeric.sum(Numeric.sum(stack.data, 2),1)
+        mcaData0 = numpy.sum(numpy.sum(stack.data, 2),1)
     else:
-        mcaData0 = Numeric.sum(Numeric.sum(stack.data, 2),0)
+        mcaData0 = numpy.sum(numpy.sum(stack.data, 2),0)
 
     import McaWindow
     mca = McaWindow.McaWidget()
@@ -602,9 +612,9 @@ if __name__ == "__main__":
             iy1 = int(ddict['xmin'])
             iy2 = int(ddict['xmax'])+1
             if imax == 0:
-                selectedData = Numeric.sum(Numeric.sum(stack.data[:,ix1:ix2, iy1:iy2], 2),1)
+                selectedData = numpy.sum(numpy.sum(stack.data[:,ix1:ix2, iy1:iy2], 2),1)
             else:
-                selectedData = Numeric.sum(Numeric.sum(stack.data[ix1:ix2,:, iy1:iy2], 2),0)
+                selectedData = numpy.sum(numpy.sum(stack.data[ix1:ix2,:, iy1:iy2], 2),0)
             sel = {}
             sel['SourceName'] = "EDF Stack"
             sel['Key'] = "Selection"
