@@ -1,34 +1,24 @@
 #/*##########################################################################
-# Copyright (C) 2004-2011 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
 #
-# This file is part of the PyMCA X-ray Fluorescence Toolkit developed at
-# the ESRF by the Beamline Instrumentation Software Support (BLISS) group.
+# This file is part of the PyMca X-ray Fluorescence Toolkit developed at
+# the ESRF by the Software group.
 #
-# This toolkit is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License as published by the Free
+# This file is free software; you can redistribute it and/or modify it
+# under the terms of the GNU Lesser General Public License as published by the Free
 # Software Foundation; either version 2 of the License, or (at your option)
 # any later version.
 #
-# PyMCA is distributed in the hope that it will be useful, but WITHOUT ANY
+# This file is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 # details.
 #
-# You should have received a copy of the GNU General Public License along with
-# PyMCA; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
-# Suite 330, Boston, MA 02111-1307, USA.
-#
-# PyMCA follows the dual licensing model of Trolltech's Qt and Riverbank's PyQt
-# and cannot be used as a free plugin for a non-free program.
-#
-# Please contact the ESRF industrial unit (industry@esrf.fr) if this license
-# is a problem for you.
 #############################################################################*/
-from numpy.oldnumeric import *
-from numpy.oldnumeric.linear_algebra import inverse
+__author__ = "V.A. Sole - ESRF Software Group"
+import numpy
+from numpy.linalg import inv
 import time
-__author__ = "V.A. Sole <sole@esrf.fr>"
-__revision__ = "$Revision: 1.19 $"
 # codes understood by the routine
 CFREE       = 0
 CPOSITIVE   = 1
@@ -46,18 +36,18 @@ def LeastSquaresFit(model, parameters0, data=None, maxiter = 100,constrains=None
                         xdata=None,ydata=None,sigmadata=None,linear=None):
     if constrains is None:
         constrains = []
-    parameters = array(parameters0).astype(Float)
+    parameters = numpy.array(parameters0).astype(numpy.float)
     if linear is None:linear=0
     if deltachi is None:
         deltachi = 0.01
     if ONED:
-      data0 = array(data)
+      data0 = numpy.array(data)
       x = data0[0:2,0]
     #import SimplePlot
     #SimplePlot.plot([data[:,0],data[:,1]],yname='Received data')
     else:
         if xdata is None:
-            x=array([y[0] for y in data])
+            x=numpy.array([y[0] for y in data])
         else:
             x=xdata
     if linear:
@@ -151,15 +141,15 @@ def LinearLeastSquaresFit(model0,parameters0,data0,maxiter,
             raise ValueError("Linear fit cannot handle quoted constraint")
     # make a local copy of the function for an easy speed up ...
     model = model0
-    parameters = array(parameters0)
+    parameters = numpy.array(parameters0)
     if data0 is not None:
-        selfx = array([x[0] for x in data0])
-        selfy = array([x[1] for x in data0])
+        selfx = numpy.array([x[0] for x in data0])
+        selfy = numpy.array([x[1] for x in data0])
     else:
         selfx = xdata
         selfy = ydata
-    selfweight = ones(selfy.shape,Float)
-    nr0 = len(selfy)
+    selfweight = numpy.ones(selfy.shape,numpy.float)
+    # nr0 = len(selfy)
     if data0 is not None:
         nc =  len(data0[0])
     else:
@@ -171,24 +161,24 @@ def LinearLeastSquaresFit(model0,parameters0,data0,maxiter,
         if nc == 3:
             #dummy = abs(data[0:nr0:inc,2])
             if data0 is not None:
-                dummy = abs(array(map(lambda x:x[2],data0)))
+                dummy = abs(numpy.array([x[2] for x in data0]))
             else:
-                dummy = abs(array(sigmadata))
-            selfweight = 1.0 / (dummy + equal(dummy,0))
+                dummy = abs(numpy.array(sigmadata))
+            selfweight = 1.0 / (dummy + numpy.equal(dummy,0))
             selfweight = selfweight * selfweight
         else:
-            selfweight = 1.0 / (abs(selfy) + equal(abs(selfy),0))
-    n_param = len(parameters)
+            selfweight = 1.0 / (abs(selfy) + numpy.equal(abs(selfy),0))
+
     #linear fit, use at own risk since there is no check for the
     #function being linear on its parameters.
     #Only the fixed constrains are handled properly
     x=selfx
     y=selfy
     weight = selfweight
-    iter  = maxiter
+    iiter  = maxiter
     niter = 0
     newpar = parameters.__copy__()
-    while (iter>0):
+    while (iiter>0):
         niter+=1
         chisq0, alpha0, beta,\
         n_free, free_index, noigno, fitparam, derivfactor  =ChisqAlphaBeta(
@@ -196,7 +186,7 @@ def LinearLeastSquaresFit(model0,parameters0,data0,maxiter,
                                                  x,y,weight,constrains,model_deriv=model_deriv,
                                                  linear=1)
         nr, nc = alpha0.shape
-        fittedpar = dot(beta, inverse(alpha0))
+        fittedpar = numpy.dot(beta, inv(alpha0))
         #check respect of constraints (only positive is handled -force parameter to 0 and fix it-)
         error = 0
         for i in range(n_free):
@@ -209,11 +199,11 @@ def LinearLeastSquaresFit(model0,parameters0,data0,maxiter,
         if error:continue
         for i in range(n_free):
             newpar[free_index[i]] = fittedpar[0,i]
-        newpar=array(getparameters(newpar,constrains))
-        iter=-1
+        newpar=numpy.array(getparameters(newpar,constrains))
+        iiter=-1
     yfit = model(newpar,x)
-    chisq = sum( weight * (y-yfit) * (y-yfit))
-    sigma0 = sqrt(abs(diagonal(inverse(alpha0))))
+    chisq = (weight * pow(y-yfit , 2)).sum()
+    sigma0 = numpy.sqrt(abs(numpy.diag(inv(alpha0))))
     sigmapar = getsigmaparameters(newpar,sigma0,constrains)
     lastdeltachi = chisq
     if not fulloutput:
@@ -265,25 +255,25 @@ def RestreinedLeastSquaresFit(model0,parameters0,data0,maxiter,
                 raise ValueError("Unknown constraint %s" % constrains[0][i])
     # make a local copy of the function for an easy speed up ...
     model = model0
-    parameters = array(parameters0)
+    parameters = numpy.array(parameters0)
     if ONED:
-        data = array(data0)
+        data = numpy.array(data0)
         x = data[1:2,0]
     fittedpar = parameters.__copy__()
     flambda = 0.001
-    iter = maxiter
+    iiter = maxiter
     niter = 0
     if ONED:
         selfx = data [:,0]
         selfy = data [:,1]
     else:
         if data0 is not None:
-            selfx = array([x[0] for x in data0])
-            selfy = array([x[1] for x in data0])
+            selfx = numpy.array([x[0] for x in data0])
+            selfy = numpy.array([x[1] for x in data0])
         else:
             selfx = xdata
             selfy = ydata
-    selfweight = ones(selfy.shape,Float)
+    selfweight = numpy.ones(selfy.shape,numpy.float)
     if ONED:
         nr0, nc = data.shape
     else:
@@ -303,23 +293,21 @@ def RestreinedLeastSquaresFit(model0,parameters0,data0,maxiter,
                     dummy = abs(data [:,2])
                 else:
                     if data0 is not None:
-                        dummy = abs(array([x[2] for x in data0]))
+                        dummy = abs(numpy.array([x[2] for x in data0]))
                     else:
-                        dummy = abs(array(sigmadata))
-                selfweight = 1.0 / (dummy + equal(dummy,0))
+                        dummy = abs(numpy.array(sigmadata))
+                selfweight = 1.0 / (dummy + numpy.equal(dummy,0))
                 selfweight = selfweight * selfweight
             else:
-                selfweight = 1.0 / (abs(selfy) + equal(abs(selfy),0))
+                selfweight = 1.0 / (abs(selfy) + numpy.equal(abs(selfy),0))
     n_param = len(parameters)
-    selfalphazeros = zeros((n_param, n_param),Float)
-    selfbetazeros = zeros((1,n_param),Float)
-    index = arange(0,nr0,2)
-    while (iter > 0):
+    index = numpy.arange(0,nr0,2)
+    while (iiter > 0):
         niter = niter + 1
         if (niter < 2) and (n_param*3 < nr0):
-                x=take(selfx,index)
-                y=take(selfy,index)
-                weight=take(selfweight,index)
+                x=numpy.take(selfx,index)
+                y=numpy.take(selfy,index)
+                weight=numpy.take(selfweight,index)
         else:
                 x=selfx
                 y=selfy
@@ -335,23 +323,23 @@ def RestreinedLeastSquaresFit(model0,parameters0,data0,maxiter,
         while flag == 0:
             newpar = parameters.__copy__()
             if(1):
-                alpha = alpha0 + flambda * identity(nr) * alpha0
-                deltapar = dot(beta, inverse(alpha))
+                alpha = alpha0 + flambda * numpy.identity(nr) * alpha0
+                deltapar = numpy.dot(beta, inv(alpha))
             else:
                 #an attempt to increase accuracy
                 #(it was unsuccessful)
-                alphadiag=sqrt(diagonal(alpha0))
-                npar = len(sqrt(diagonal(alpha0)))
-                narray = zeros((npar,npar),Float)
+                alphadiag=numpy.sqrt(numpy.diag(alpha0))
+                npar = len(numpy.sqrt(alphadiag))
+                narray = numpy.zeros((npar,npar),numpy.float)
                 for i in range(npar):
                     for j in range(npar):
                         narray[i,j] = alpha0[i,j]/(alphadiag[i]*alphadiag[j])
-                narray = inverse(narray + flambda * identity(nr))
+                narray = inv(narray + flambda * numpy.identity(nr))
                 for i in range(npar):
                     for j in range(npar):
                         narray[i,j] = narray[i,j]/(alphadiag[i]*alphadiag[j])
-                deltapar = dot(beta, narray)
-            pwork = zeros(deltapar.shape, Float)
+                deltapar = numpy.dot(beta, narray)
+            pwork = numpy.zeros(deltapar.shape, numpy.float)
             for i in range(n_free):
                 if constrains [0] [free_index[i]] == CFREE:
                     pwork [0] [i] = fitparam [i] + deltapar [0] [i]
@@ -359,8 +347,8 @@ def RestreinedLeastSquaresFit(model0,parameters0,data0,maxiter,
                     #abs method
                     pwork [0] [i] = fitparam [i] + deltapar [0] [i]
                     #square method
-                    #pwork [0] [i] = (sqrt(fitparam [i]) + deltapar [0] [i]) * \
-                    #                (sqrt(fitparam [i]) + deltapar [0] [i])
+                    #pwork [0] [i] = (numpy.sqrt(fitparam [i]) + deltapar [0] [i]) * \
+                    #                (numpy.sqrt(fitparam [i]) + deltapar [0] [i])
                 elif constrains [0] [free_index[i]] == CQUOTED:
                     pmax=max(constrains[1] [free_index[i]],
                             constrains[2] [free_index[i]])
@@ -370,34 +358,34 @@ def RestreinedLeastSquaresFit(model0,parameters0,data0,maxiter,
                     B = 0.5 * (pmax - pmin)
                     if (B != 0):
                         pwork [0] [i] = A + \
-                                    B * sin(arcsin((fitparam[i] - A)/B)+ \
+                                    B * numpy.sin(numpy.arcsin((fitparam[i] - A)/B)+ \
                                     deltapar [0] [i])
                     else:
                         print("Error processing constrained fit")
                         print("Parameter limits are",pmin,' and ',pmax)
                         print("A = ",A,"B = ",B)
                 newpar [free_index[i]] = pwork [0] [i]
-            newpar=array(getparameters(newpar,constrains))
-            workpar = take(newpar,noigno)
+            newpar=numpy.array(getparameters(newpar,constrains))
+            workpar = numpy.take(newpar,noigno)
             #yfit = model(workpar.tolist(), x)
             yfit = model(workpar,x)
-            chisq = sum( weight * (y-yfit) * (y-yfit))
+            chisq = (weight * pow(y-yfit, 2)).sum()
             if chisq > chisq0:
                 flambda = flambda * 10.0
                 if flambda > 1000:
                     flag = 1
-                    iter = 0
+                    iiter = 0
             else:
                 flag = 1
                 fittedpar = newpar.__copy__()
                 lastdeltachi = (chisq0-chisq)/(chisq0+(chisq0==0))
                 if (lastdeltachi) < deltachi:
-                    iter = 0
+                    iiter = 0
                 chisq0 = chisq
                 flambda = flambda / 10.0
                 #print "iter = ",iter,"chisq = ", chisq
-            iter = iter -1
-    sigma0 = sqrt(abs(diagonal(inverse(alpha0))))
+            iiter = iiter -1
+    sigma0 = numpy.sqrt(abs(numpy.diag(inv(alpha0))))
     sigmapar = getsigmaparameters(fittedpar,sigma0,constrains)
     if not fulloutput:
         return fittedpar.tolist(), chisq/(len(yfit)-len(sigma0)), sigmapar.tolist()
@@ -425,8 +413,8 @@ def ChisqAlphaBeta(model0, parameters, x,y,weight, constrains,model_deriv=None,l
         elif constrains[0] [i] == CPOSITIVE:
             fitparam.append(abs(parameters[i]))
             derivfactor.append(1.0)
-            #fitparam.append(sqrt(abs(parameters[i])))
-            #derivfactor.append(2.0*sqrt(abs(parameters[i])))
+            #fitparam.append(numpy.sqrt(abs(parameters[i])))
+            #derivfactor.append(2.0*numpy.sqrt(abs(parameters[i])))
             free_index.append(i)
             n_free += 1
         elif constrains[0] [i] == CQUOTED:
@@ -439,11 +427,11 @@ def ChisqAlphaBeta(model0, parameters, x,y,weight, constrains,model_deriv=None,l
                 B = 0.5 * (pmax - pmin)
                 if 1:
                     fitparam.append(parameters[i])
-                    derivfactor.append(B*cos(arcsin((parameters[i] - A)/B)))
+                    derivfactor.append(B*numpy.cos(numpy.arcsin((parameters[i] - A)/B)))
                 else:
-                    help0 = arcsin((parameters[i] - A)/B)
+                    help0 = numpy.arcsin((parameters[i] - A)/B)
                     fitparam.append(help0)
-                    derivfactor.append(B*cos(help0))
+                    derivfactor.append(B*numpy.cos(help0))
                 free_index.append(i)
                 n_free += 1
             elif (pmax-pmin) > 0:
@@ -451,10 +439,10 @@ def ChisqAlphaBeta(model0, parameters, x,y,weight, constrains,model_deriv=None,l
                 print("Initial value = %f" % parameters[i])
                 print("Limits are %f and %f" % (pmax, pmin))
                 print("Parameter will be kept at its starting value")
-    fitparam = array(fitparam, Float)
-    alpha = zeros((n_free, n_free),Float)
-    beta = zeros((1,n_free),Float)
-    delta = (fitparam + equal(fitparam,0.0)) * 0.00001
+    fitparam = numpy.array(fitparam, numpy.float)
+    alpha = numpy.zeros((n_free, n_free),numpy.float)
+    beta = numpy.zeros((1,n_free),numpy.float)
+    delta = (fitparam + numpy.equal(fitparam,0.0)) * 0.00001
     nr  = x.shape[0]
     ##############
     # Prior to each call to the function one has to re-calculate the
@@ -463,23 +451,23 @@ def ChisqAlphaBeta(model0, parameters, x,y,weight, constrains,model_deriv=None,l
     for i in range(n_free):
         pwork [free_index[i]] = fitparam [i]
     newpar = getparameters(pwork.tolist(),constrains)
-    newpar = take(newpar,noigno)
+    newpar = numpy.take(newpar,noigno)
     for i in range(n_free):
         if model_deriv is None:
             #pwork = parameters.__copy__()
             pwork [free_index[i]] = fitparam [i] + delta [i]
             newpar = getparameters(pwork.tolist(),constrains)
-            newpar=take(newpar,noigno)
+            newpar=numpy.take(newpar,noigno)
             f1 = model(newpar, x)
             pwork [free_index[i]] = fitparam [i] - delta [i]
             newpar = getparameters(pwork.tolist(),constrains)
-            newpar=take(newpar,noigno)
+            newpar=numpy.take(newpar,noigno)
             f2 = model(newpar, x)
             help0 = (f1-f2) / (2.0 * delta [i])
             help0 = help0 * derivfactor[i]
             pwork [free_index[i]] = fitparam [i]
             #removed I resize outside the loop:
-            #help0 = resize(help0,(1,nr))
+            #help0 = numpy.resize(help0,(1,nr))
         else:
             newpar = getparameters(pwork.tolist(),constrains)
             help0=model_deriv(pwork,free_index[i],x)
@@ -488,9 +476,9 @@ def ChisqAlphaBeta(model0, parameters, x,y,weight, constrains,model_deriv=None,l
         if i == 0 :
             deriv = help0
         else:
-            deriv = concatenate ((deriv,help0), 0)
+            deriv = numpy.concatenate((deriv,help0), 0)
     #line added to resize outside the loop
-    deriv=resize(deriv,(n_free,nr))
+    deriv=numpy.resize(deriv,(n_free,nr))
     if linear:
         pseudobetahelp = weight * y
     else:
@@ -498,28 +486,28 @@ def ChisqAlphaBeta(model0, parameters, x,y,weight, constrains,model_deriv=None,l
         deltay = y - yfit
         help0 = weight * deltay
     for i in range(n_free):
-        derivi = resize(deriv [i,:], (1,nr))
+        derivi = numpy.resize(deriv [i,:], (1,nr))
         if linear:
             if i==0:
-                beta = resize(sum((pseudobetahelp * derivi),1),(1,1))
+                beta = numpy.resize(numpy.sum((pseudobetahelp * derivi),1),(1,1))
             else:
-                beta = concatenate((beta, resize(sum((pseudobetahelp * derivi),1),(1,1))), 1)
+                beta = numpy.concatenate((beta, numpy.resize(numpy.sum((pseudobetahelp * derivi),1),(1,1))), 1)
         else:
-            help1 = resize(sum((help0 * derivi),1),(1,1))
+            help1 = numpy.resize(numpy.sum((help0 * derivi),1),(1,1))
             if i == 0:
                 beta = help1
             else:
-                beta = concatenate ((beta, help1), 1)
-        help1 = innerproduct(deriv,weight*derivi)
+                beta = numpy.concatenate((beta, help1), 1)
+        help1 = numpy.inner(deriv,weight*derivi)
         if i == 0:
             alpha = help1
         else:
-            alpha = concatenate((alpha, help1),1)
+            alpha = numpy.concatenate((alpha, help1),1)
     if linear:
         #not used
         chisq = 0.0
     else:
-        chisq = sum(help0 * deltay)
+        chisq = (help0 * deltay).sum()
     return chisq, alpha, beta, \
            n_free, free_index, noigno, fitparam, derivfactor
 
@@ -543,7 +531,7 @@ def getparameters(parameters,constrains):
                 pmin=min(constrains[1] [i],constrains[2] [i])
                 A = 0.5 * (pmax + pmin)
                 B = 0.5 * (pmax - pmin)
-                newparam.append(A + B * sin(parameters[i]))
+                newparam.append(A + B * numpy.sin(parameters[i]))
         elif abs(constrains[0][i]) == CFIXED:
             newparam.append(parameters[i])
         else:
@@ -563,7 +551,7 @@ def getsigmaparameters(parameters,sigma0,constrains):
     # 0 = Free       1 = Positive     2 = Quoted
     # 3 = Fixed      4 = Factor       5 = Delta
     n_free = 0
-    sigma_par = zeros(parameters.shape,Float)
+    sigma_par = numpy.zeros(parameters.shape,numpy.float)
     for i in range(len(constrains [0])):
         if constrains[0][i] == CFREE:
             sigma_par [i] = sigma0[n_free]
@@ -575,10 +563,10 @@ def getsigmaparameters(parameters,sigma0,constrains):
         elif constrains[0][i] == CQUOTED:
             pmax = max(constrains [1] [i], constrains [2] [i])
             pmin = min(constrains [1] [i], constrains [2] [i])
-            A = 0.5 * (pmax + pmin)
+            # A = 0.5 * (pmax + pmin)
             B = 0.5 * (pmax - pmin)
             if (B > 0) & (parameters [i] < pmax) & (parameters [i] > pmin):
-                sigma_par [i] = abs(B) * cos(parameters[i]) * sigma0[n_free]
+                sigma_par [i] = abs(B * numpy.cos(parameters[i]) * sigma0[n_free])
                 n_free += 1
             else:
                 sigma_par [i] = parameters[i]
@@ -605,30 +593,30 @@ def fitpar2par(fitpar,constrains,free_index):
             pmin=min(constrains[1] [free_index[i]],constrains[2] [free_index[i]])
             A = 0.5 * (pmax + pmin)
             B = 0.5 * (pmax - pmin)
-            newparam.append(A + B * sin(fitpar[i]))
+            newparam.append(A + B * numpy.sin(fitpar[i]))
     return newparam
 
 def gauss(param0,t0):
-    param=array(param0)
-    t=array(t0)
+    param=numpy.numpy.array(param0)
+    t=numpy.numpy.array(t0)
     dummy=2.3548200450309493*(t-param[3])/param[4]
     return param[0] + param[1] * t + param[2] * myexp(-0.5 * dummy * dummy)
 
 def myexp(x):
     # put a (bad) filter to avoid over/underflows
     # with no python looping
-    return exp(x*less(abs(x),250))-1.0*greater_equal(abs(x),250)
+    return numpy.exp(x*numpy.less(abs(x),250))-1.0 * numpy.greater_equal(abs(x),250)
 
 
 def test(npoints):
     xx = arange (npoints)
-    xx=resize(xx,(npoints,1))
+    xx=numpy.resize(xx,(npoints,1))
     #yy = 1000.0 * exp (- 0.5 * (xx * xx) /15)+ 2.0 * xx + 10.5
     yy = gauss([10.5,2,1000.0,20.,15],xx)
-    yy=resize(yy,(npoints,1))
-    sy = sqrt(abs(yy))
-    sy=resize(sy,(npoints,1))
-    data = concatenate((xx, yy, sy),1)
+    yy=numpy.resize(yy,(npoints,1))
+    sy = numpy.sqrt(abs(yy))
+    sy=numpy.resize(sy,(npoints,1))
+    data = numpy.concatenate((xx, yy, sy),1)
     parameters = [0.0,1.0,900.0, 25., 10]
     stime = time.time()
     if 0:
