@@ -1,92 +1,101 @@
 #/*##########################################################################
 # Copyright (C) 2004-2012 European Synchrotron Radiation Facility
 #
-# This file is part of the PyMCA X-ray Fluorescence Toolkit developed at
-# the ESRF by the Beamline Instrumentation Software Support (BLISS) group.
+# This file is part of the PyMca X-ray Fluorescence Toolkit developed at
+# the ESRF by the Software group.
 #
-# This toolkit is free software; you can redistribute it and/or modify it 
+# This toolkit is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the Free
-# Software Foundation; either version 2 of the License, or (at your option) 
+# Software Foundation; either version 2 of the License, or (at your option)
 # any later version.
 #
-# PyMCA is distributed in the hope that it will be useful, but WITHOUT ANY
+# PyMca is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 # FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 # details.
 #
 # You should have received a copy of the GNU General Public License along with
-# PyMCA; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
-# Suite 330, Boston, MA 02111-1307, USA.
+# PyMca; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
-# PyMCA follows the dual licensing model of Trolltech's Qt and Riverbank's PyQt
-# and cannot be used as a free plugin for a non-free program. 
+# PyMca follows the dual licensing model of Riverbank's PyQt and cannot be
+# used as a free plugin for a non-free program.
 #
-# Please contact the ESRF industrial unit (industry@esrf.fr) if this license 
+# Please contact the ESRF industrial unit (industry@esrf.fr) if this license
 # is a problem for you.
 #############################################################################*/
 __author__ = "V.A. Sole - ESRF BLISS Group"
 import sys
-import RGBCorrelatorGraph
+import os
+import numpy
+from PyMca import RGBCorrelatorGraph
 qt = RGBCorrelatorGraph.qt
 IconDict = RGBCorrelatorGraph.IconDict
-QWTVERSION4 = RGBCorrelatorGraph.QtBlissGraph.QWTVERSION4
 QTVERSION = qt.qVersion()
 if hasattr(qt, "QString"):
     QString = qt.QString
 else:
-    QString = str
+    QString = qt.safe_str
 MATPLOTLIB = False
 if QTVERSION > '4.0.0':
     try:
-        import QPyMcaMatplotlibSave
+        from PyMca import QPyMcaMatplotlibSave
         MATPLOTLIB = True
     except ImportError:
         MATPLOTLIB = False
 else:
     qt.QIcon = qt.QIconSet
-import numpy
-import ColormapDialog
-import spslut
-import os
-import PyMcaDirs
-import ArraySave
+from PyMca import ColormapDialog
+from PyMca import spslut
+from PyMca import PyMcaDirs
+from PyMca import ArraySave
 try:
     from PyMca import ProfileScanWidget
 except ImportError:
+    print("MaskImageWidget importing ProfileScanWidget directly")
     import ProfileScanWidget
 try:
     from PyMca import SpecfitFuns
 except ImportError:
+    print("MaskImageWidget importing SpecfitFuns directly")
     import SpecfitFuns
 
 
 COLORMAPLIST = [spslut.GREYSCALE, spslut.REVERSEGREY, spslut.TEMP,
                 spslut.RED, spslut.GREEN, spslut.BLUE, spslut.MANY]
 
-if QWTVERSION4:
-    raise ImportError("MaskImageWidget needs Qwt5")
-
 if QTVERSION > '4.0.0':
-    import PyQt4.Qwt5 as Qwt
+    from PyQt4 import Qwt5
 else:
-    import Qwt5 as Qwt
+    import Qwt5
 
 try:
-    import QwtPlotItems
+    from PyMca import QwtPlotItems
     OVERLAY_DRAW = True
 except ImportError:
     OVERLAY_DRAW = False
 
 DEBUG = 0
 
-if QTVERSION < '4.6.0':
-    USE_PICKER = True
-else:
+
+# set this variable to false if you get crashes when moving the mouse
+# over the images.
+# Before I thought it had to do with the Qt version used, but it seems
+# to be related to old PyQwt versions. (In fact, the 5.2.1 version is
+# a recent snapshot)
+if Qwt5.QWT_VERSION_STR < '5.2.1':
     USE_PICKER = False
-class MyPicker(Qwt.QwtPlotPicker):
+else:
+    USE_PICKER = True
+
+# Uncomment next line if you experience crashes moving the mouse on
+# top of the images
+#USE_PICKER = True
+
+class MyPicker(Qwt5.QwtPlotPicker):
     def __init__(self, *var):
-        Qwt.QwtPlotPicker.__init__(self, *var)
-        self.__text = Qwt.QwtText()
+        Qwt5.QwtPlotPicker.__init__(self, *var)
+        self.__text = Qwt5.QwtText()
         self.data = None
 
     if USE_PICKER:
@@ -96,8 +105,8 @@ class MyPicker(Qwt.QwtPlotPicker):
                 self.__text.setText("%g, %g" % (d.x(), d.y()))
             else:
                 limits = self.data.shape
-                x = round(d.y())
-                y = round(d.x())
+                x = int(d.y())
+                y = int(d.x())
                 if x < 0: x = 0
                 if y < 0: y = 0
                 x = min(int(x), limits[0]-1)
@@ -214,11 +223,11 @@ class MaskImageWidget(qt.QWidget):
         self.connect(self.graphWidget.zoomResetToolButton,
                      qt.SIGNAL("clicked()"), 
                      self._zoomResetSignal)
-        self.graphWidget.picker = MyPicker(Qwt.QwtPlot.xBottom,
-                           Qwt.QwtPlot.yLeft,
-                           Qwt.QwtPicker.NoSelection,
-                           Qwt.QwtPlotPicker.CrossRubberBand,
-                           Qwt.QwtPicker.AlwaysOn,
+        self.graphWidget.picker = MyPicker(Qwt5.QwtPlot.xBottom,
+                           Qwt5.QwtPlot.yLeft,
+                           Qwt5.QwtPicker.NoSelection,
+                           Qwt5.QwtPlotPicker.CrossRubberBand,
+                           Qwt5.QwtPicker.AlwaysOn,
                            self.graphWidget.graph.canvas())
         self.graphWidget.picker.setTrackerPen(qt.QPen(qt.Qt.black))
         self.graphWidget.graph.enableSelection(False)
@@ -378,7 +387,7 @@ class MaskImageWidget(qt.QWidget):
         try:
             title = self.graphWidget.graph.title().text()
             if sys.version < '3.0':
-                title = str(title)
+                title = qt.safe_str(title)
         except:
             title = ""
         return title
@@ -412,6 +421,7 @@ class MaskImageWidget(qt.QWidget):
         shape = imageData.shape
         width = ddict['pixelwidth'] - 1
         if ddict['mode'].upper() in ["HLINE", "HORIZONTAL"]:
+            deltaDistance = 1.0
             if width < 1:
                 row = int(ddict['y'][0])
                 if row < 0:
@@ -450,6 +460,7 @@ class MaskImageWidget(qt.QWidget):
                                          replot=True)
             xdata  = numpy.arange(shape[1]).astype(numpy.float)
         elif ddict['mode'].upper() in ["VLINE", "VERTICAL"]:
+            deltaDistance = 1.0
             if width < 1:
                 column = int(ddict['x'][0])
                 if column < 0:
@@ -503,6 +514,9 @@ class MaskImageWidget(qt.QWidget):
                 npoints = deltaCol+1
             else:    
                 npoints = deltaRow+1
+            #get the abscisa in distance units
+            deltaDistance = numpy.sqrt(float(deltaCol)*deltaCol +
+                                       float(deltaRow)*deltaRow)/(npoints-1.0)
             if width < 1:
                 x = numpy.zeros((npoints, 2), numpy.float)
                 x[:, 0] = numpy.linspace(row0, row1, npoints)
@@ -511,6 +525,7 @@ class MaskImageWidget(qt.QWidget):
                 #perform the interpolation
                 ydata = self._interpolate((x0, y0), imageData, x)
                 xdata = numpy.arange(float(npoints))
+                
                 if overlay:
                     #self.drawOverlayItem(x, y, legend=name, info=info, replot, replace)
                     self.drawOverlayItem([col0, col1],
@@ -739,9 +754,14 @@ class MaskImageWidget(qt.QWidget):
                 print("Mode %s not supported yet" % ddict['mode'])
             return
         info = {}
-        info['xlabel'] = "Point"
         info['ylabel'] = "Z"
-        return xdata, ydata, legend, info
+        if 0:
+            info['xlabel'] = "Point"
+            return xdata, ydata, legend, info
+        else:
+            #users prefer to work with distance
+            info['xlabel'] = "Distance"
+            return xdata * deltaDistance, ydata, legend, info
 
     def _profileSelectionSlot(self, ddict):
         if DEBUG:
@@ -823,9 +843,6 @@ class MaskImageWidget(qt.QWidget):
         self.__lastOverlayLegend = legend
 
     def _hFlipIconSignal(self):
-        if QWTVERSION4:
-            qt.QMessageBox.information(self, "Flip Image", "Not available under PyQwt4")
-            return
         if not self.graphWidget.graph.yAutoScale:
             qt.QMessageBox.information(self, "Open",
                     "Please set Y Axis to AutoScale first")
@@ -892,7 +909,7 @@ class MaskImageWidget(qt.QWidget):
             print("_setEraseSelectionMode")
         self.__eraseMode = True
         self.__brushMode = True
-        self.graphWidget.picker.setTrackerMode(Qwt.QwtPicker.ActiveOnly)
+        self.graphWidget.picker.setTrackerMode(Qwt5.QwtPicker.ActiveOnly)
         self.graphWidget.graph.enableSelection(False)
 
     def _setRectSelectionMode(self):
@@ -900,7 +917,7 @@ class MaskImageWidget(qt.QWidget):
             print("_setRectSelectionMode")
         self.__eraseMode = False
         self.__brushMode = False
-        self.graphWidget.picker.setTrackerMode(Qwt.QwtPicker.AlwaysOn)
+        self.graphWidget.picker.setTrackerMode(Qwt5.QwtPicker.AlwaysOn)
         self.graphWidget.graph.enableSelection(True)
         
     def _setBrushSelectionMode(self):
@@ -908,7 +925,7 @@ class MaskImageWidget(qt.QWidget):
             print("_setBrushSelectionMode")
         self.__eraseMode = False
         self.__brushMode = True
-        self.graphWidget.picker.setTrackerMode(Qwt.QwtPicker.ActiveOnly)
+        self.graphWidget.picker.setTrackerMode(Qwt5.QwtPicker.ActiveOnly)
         self.graphWidget.graph.enableSelection(False)
         
     def _setBrush(self):
@@ -979,7 +996,7 @@ class MaskImageWidget(qt.QWidget):
         if mode:
             self.graphWidget.graph.enableSelection(True)
             self.__brushMode  = False
-            self.graphWidget.picker.setTrackerMode(Qwt.QwtPicker.AlwaysOn)
+            self.graphWidget.picker.setTrackerMode(Qwt5.QwtPicker.AlwaysOn)
             if QTVERSION < '4.0.0':
                 self.graphWidget.selectionToolButton.setState(qt.QButton.On)
             else:
@@ -989,7 +1006,7 @@ class MaskImageWidget(qt.QWidget):
             self.graphWidget.selectionToolButton.setDown(True)
             self.graphWidget.showImageIcons()            
         else:
-            self.graphWidget.picker.setTrackerMode(Qwt.QwtPicker.AlwaysOff)
+            self.graphWidget.picker.setTrackerMode(Qwt5.QwtPicker.AlwaysOff)
             self.graphWidget.showProfileSelectionIcons()
             self.graphWidget.graph.enableZoom(True)
             if QTVERSION < '4.0.0':
@@ -1023,18 +1040,18 @@ class MaskImageWidget(qt.QWidget):
                                              numpy.uint8)
         minValue, maxValue = self._getSelectionMinMax()
         tmpData = numpy.array(self.__imageData, copy=True)
-        tmpData[numpy.isfinite(self.__imageData)] = minValue
+        tmpData[True - numpy.isfinite(self.__imageData)] = minValue
         selectionMask[tmpData >= maxValue] = 1
         self.setSelectionMask(selectionMask, plot=False)
         self.plotImage(update=False)
         self._emitMaskChangedSignal()
         
     def _selectMiddle(self):
-        selectionMask = numpy.zeros(self.__imageData.shape,
+        selectionMask = numpy.ones(self.__imageData.shape,
                                              numpy.uint8)
         minValue, maxValue = self._getSelectionMinMax()
         tmpData = numpy.array(self.__imageData, copy=True)
-        tmpData[numpy.isfinite(self.__imageData)] = maxValue
+        tmpData[True - numpy.isfinite(self.__imageData)] = maxValue
         selectionMask[tmpData >= maxValue] = 0
         selectionMask[tmpData <= minValue] = 0
         self.setSelectionMask(selectionMask, plot=False)
@@ -1046,7 +1063,7 @@ class MaskImageWidget(qt.QWidget):
                                              numpy.uint8)
         minValue, maxValue = self._getSelectionMinMax()
         tmpData = numpy.array(self.__imageData, copy=True)
-        tmpData[numpy.isfinite(self.__imageData)] = maxValue
+        tmpData[True - numpy.isfinite(self.__imageData)] = maxValue
         selectionMask[tmpData <= minValue] = 1
         self.setSelectionMask(selectionMask, plot=False)
         self.plotImage(update=False)
@@ -1561,9 +1578,6 @@ class MaskImageWidget(qt.QWidget):
                         qt.SIGNAL('MaskImageWidgetSignal'),
                         ddict)
 
-    def _saveToolButtonSignal(self):
-        self._saveMenu.exec_(self.cursor().pos())
-
     def _zoomResetSignal(self):
         if DEBUG:
             print("_zoomResetSignal")
@@ -1599,9 +1613,9 @@ class MaskImageWidget(qt.QWidget):
         if not ret: return ""
         filename = filedialog.selectedFiles()[0]
         if len(filename):
-            filename = str(filename)
+            filename = qt.safe_str(filename)
             self.outputDir = os.path.dirname(filename)
-            self._saveFilter = str(filedialog.selectedFilter())
+            self._saveFilter = qt.safe_str(filedialog.selectedFilter())
             filterused = "."+self._saveFilter[-3:]
             PyMcaDirs.outputDir = os.path.dirname(filename)
             if len(filename) < 4:
@@ -1744,8 +1758,8 @@ def test():
 
     if QTVERSION < '4.0.0':
         qt.QObject.connect(container,
-                       qt.PYSIGNAL("MaskImageWidgetSignal"),
-                       updateMask)
+                           qt.PYSIGNAL("MaskImageWidgetSignal"),
+                           theSlot)
         app.setMainWidget(container)
         app.exec_loop()
     else:
@@ -1756,4 +1770,4 @@ def test():
 
 if __name__ == "__main__":
     test()
-        
+
