@@ -1,52 +1,51 @@
 #/*##########################################################################
-# Copyright (C) 2004-2011 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
 #
-# This file is part of the PyMCA X-ray Fluorescence Toolkit developed at
-# the ESRF by the Beamline Instrumentation Software Support (BLISS) group.
+# This file is part of the PyMca X-ray Fluorescence Toolkit developed at
+# the ESRF by the Software group.
 #
-# This toolkit is free software; you can redistribute it and/or modify it 
+# This toolkit is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the Free
-# Software Foundation; either version 2 of the License, or (at your option) 
+# Software Foundation; either version 2 of the License, or (at your option)
 # any later version.
 #
-# PyMCA is distributed in the hope that it will be useful, but WITHOUT ANY
+# PyMca is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 # FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 # details.
 #
 # You should have received a copy of the GNU General Public License along with
-# PyMCA; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
-# Suite 330, Boston, MA 02111-1307, USA.
+# PyMca; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
-# PyMCA follows the dual licensing model of Trolltech's Qt and Riverbank's PyQt
-# and cannot be used as a free plugin for a non-free program. 
+# PyMca follows the dual licensing model of Riverbank's PyQt and cannot be
+# used as a free plugin for a non-free program.
 #
-# Please contact the ESRF industrial unit (industry@esrf.fr) if this license 
+# Please contact the ESRF industrial unit (industry@esrf.fr) if this license
 # is a problem for you.
 #############################################################################*/
 __revision__ = "$Revision: 1.32 $"
-import ClassMcaTheory
-import SpecFileLayer
-import EdfFileLayer
-import EdfFile
-import LuciaMap
-import AifiraMap
-import EDFStack
+import sys
+import os
+import numpy
+from PyMca import ClassMcaTheory
+from PyMca import SpecFileLayer
+from PyMca import EdfFileLayer
+from PyMca import EdfFile
+from PyMca import LuciaMap
+from PyMca import AifiraMap
+from PyMca import EDFStack
 try:
     import h5py
-    import HDF5Stack1D
+    from PyMca import HDF5Stack1D
     HDF5SUPPORT = True
 except ImportError:
     HDF5SUPPORT = False
-import numpy
-import numpy.oldnumeric as Numeric
-import os
-import sys
-import ConfigDict
-import ConcentrationsTool
+from PyMca import ConfigDict
+from PyMca import ConcentrationsTool
 
 
-class McaAdvancedFitBatch:
+class McaAdvancedFitBatch(object):
     def __init__(self,initdict,filelist=None,outputdir=None,
                     roifit=None,roiwidth=None,
                     overwrite=1, filestep=1, mcastep=1,
@@ -306,8 +305,8 @@ class McaAdvancedFitBatch:
                     xmin = 0.0
                 #key = "%s.%s.%02d.%02d" % (scan,order,row,col)
                 key = "%s.%04d" % (keylist[i], mca)
-                y0  = Numeric.array(mcadata)
-                x = Numeric.arange(len(y0))*1.0 + xmin
+                y0  = numpy.array(mcadata)
+                x = numpy.arange(len(y0))*1.0 + xmin
                 #I only process the first file of the stack?
                 filename = os.path.basename(info['SourceName'][0])
                 infoDict = {}
@@ -321,18 +320,13 @@ class McaAdvancedFitBatch:
     def __processOneFile(self):
         ffile=self.file
         fileinfo = ffile.GetSourceInfo()
-        nimages = nscans = len(fileinfo['KeyList'])
         if 1:
             i = 0
             for scankey in  fileinfo['KeyList']:
                 if self.pleaseBreak: break
                 self.onImage(scankey, fileinfo['KeyList'])
-                if 0:
-                    scan,rc   = string.split(scankey,".")
-                    info,data  = ffile.LoadSource({'Key':int(image)-1})
-                else:
-                    scan,order = scankey.split(".")
-                    info,data  = ffile.LoadSource(scankey)
+                scan,order = scankey.split(".")
+                info,data  = ffile.LoadSource(scankey)
                 if info['SourceType'] == "EdfFile":
                     nrows = int(info['Dim_1'])
                     ncols = int(info['Dim_2'])
@@ -344,26 +338,16 @@ class McaAdvancedFitBatch:
                         if self.pleaseBreak: break
                         self.__col += 1
                         if int(nrows) > int(ncols):
-                            row=mca
-                            col=0
                             mcadata = data[mca,:]
                         else:
-                            col=mca
-                            row=0
                             mcadata = data[:,mca]
                         if 'MCA start ch' in info:
                             xmin = float(info['MCA start ch'])
                         else:
                             xmin = 0.0
-                        #key = "%s.%s.%02d.%02d" % (scan,order,row,col)
                         key = "%s.%s.%04d" % (scan,order,mca)
-                        if 0:
-                            #slow
-                            y0  = Numeric.array(mcadata.tolist())
-                        else:
-                            #fast
-                            y0  = Numeric.array(mcadata)
-                        x = Numeric.arange(len(y0))*1.0 + xmin
+                        y0  = numpy.array(mcadata)
+                        x = numpy.arange(len(y0))*1.0 + xmin
                         filename = os.path.basename(info['SourceName'])
                         infoDict = {}
                         infoDict['SourceName'] = info['SourceName']
@@ -387,7 +371,7 @@ class McaAdvancedFitBatch:
                         #situations
                         #if self.__row == 0:
                         if self.counter == 0:
-                            self.__chann0List = Numeric.zeros(info['NbMcaDet'])
+                            self.__chann0List = numpy.zeros(info['NbMcaDet'])
                             chan0list = scan_obj.header('@CHANN')
                             if len(chan0list):
                                 for i in range(info['NbMcaDet']):
@@ -404,8 +388,8 @@ class McaAdvancedFitBatch:
                             #get rid of slow info reading methods
                             #mcainfo,mcadata = ffile.LoadSource(key)
                             mcadata = scan_obj.mca(i+1)
-                            y0  = Numeric.array(mcadata)
-                            x = Numeric.arange(len(y0))*1.0 + \
+                            y0  = numpy.array(mcadata)
+                            x = numpy.arange(len(y0))*1.0 + \
                                 self.__chann0List[mca-1]
                             filename = os.path.basename(info['SourceName'])
 
@@ -419,26 +403,46 @@ class McaAdvancedFitBatch:
                             #print "remaining = ",(time.time()-e0) * (info['NbMca'] - i)
 
     def __getFitFile(self, filename, key):
-        fitdir = os.path.join(self._outputdir,"FIT")
-        fitdir = os.path.join(fitdir,filename+"_FITDIR")
+        fitdir = self.os_path_join(self._outputdir,"FIT")
+        fitdir = self.os_path_join(fitdir,filename+"_FITDIR")
         outfile = filename +"_"+key+".fit" 
-        outfile = os.path.join(fitdir,  outfile)           
+        outfile = self.os_path_join(fitdir,  outfile)           
+        return outfile
+
+    def os_path_join(self, a, b):
+        try:
+            outfile=os.path.join(a, b)
+        except UnicodeDecodeError:
+            toBeDone = True
+            if sys.platform == 'win32':
+                try:
+                    outfile=os.path.join(a.decode('mbcs'),
+                                         b.decode('mbcs'))
+                    toBeDone = False
+                except UnicodeDecodeError:
+                    pass
+            if toBeDone:
+                try:
+                    outfile = os.path.join(a.decode('utf-8'),
+                                           a.decode('utf-8'))
+                except UnicodeDecodeError:
+                    outfile = os.path.join(a.decode('latin-1'),
+                                           a.decode('latin-1'))
         return outfile
 
-                
     def __processOneMca(self,x,y,filename,key,info=None):
         self._concentrationsAsAscii = ""
         if not self.roiFit:
             result = None
             concentrationsdone = 0
             concentrations = None
-            outfile=os.path.join(self._outputdir,filename)
+            outfile=self.os_path_join(self._outputdir, filename)
             fitfile = self.__getFitFile(filename,key)
             if self.chunk is not None:
                 con_extension = "_%06d_partial_concentrations.txt" % self.chunk
             else:
                 con_extension = "_concentrations.txt"
-            self._concentrationsFile = os.path.join(self._outputdir,
+            self._concentrationsFile = self.os_path_join(self._outputdir,
                                     self._rootname+ con_extension)
             #                        self._rootname+"_concentrationsNEW.txt")
             if self.counter == 0:
@@ -467,7 +471,7 @@ class McaAdvancedFitBatch:
                 try:
                     #I make sure I take the fit limits configuration
                     self.mcafit.config['fit']['use_limit'] = 1
-                    self.mcafit.setdata(x,y)
+                    self.mcafit.setData(x,y)
                 except:
                     print("Error entering data of file with output = %s" %\
                           filename)
@@ -550,14 +554,14 @@ class McaAdvancedFitBatch:
             #output options
             # .FIT files
             if self.fitFiles:
-                fitdir = os.path.join(self._outputdir,"FIT")
+                fitdir = self.os_path_join(self._outputdir,"FIT")
                 if not os.path.exists(fitdir):
                     try:
                         os.mkdir(fitdir)
                     except:
                         print("I could not create directory %s" % fitdir)
                         return
-                fitdir = os.path.join(fitdir,filename+"_FITDIR")
+                fitdir = self.os_path_join(fitdir,filename+"_FITDIR")
                 if not os.path.exists(fitdir):
                     try:
                         os.mkdir(fitdir)
@@ -568,7 +572,7 @@ class McaAdvancedFitBatch:
                     print("%s does not seem to be a valid directory" % fitdir)
                 else:
                     outfile = filename +"_"+key+".fit" 
-                    outfile = os.path.join(fitdir,  outfile)
+                    outfile = self.os_path_join(fitdir,  outfile)
                 if not useExistingResult:
                     result = self.mcafit.digestresult(outfile=outfile,
                                                       info=info)
@@ -589,7 +593,7 @@ class McaAdvancedFitBatch:
                 #python like output list
                 if not self.counter:
                     name = os.path.splitext(self._rootname)[0]+"_fitfilelist.py"
-                    name = os.path.join(self._outputdir,name)
+                    name = self.os_path_join(self._outputdir,name)
                     try:
                         os.remove(name)
                     except:
@@ -612,7 +616,7 @@ class McaAdvancedFitBatch:
                 #this only works with EDF
                 if self.__ncols is not None:
                     if not self.counter:
-                        imgdir = os.path.join(self._outputdir,"IMAGES")
+                        imgdir = self.os_path_join(self._outputdir,"IMAGES")
                         if not os.path.exists(imgdir):
                             try:
                                 os.mkdir(imgdir)
@@ -631,9 +635,15 @@ class McaAdvancedFitBatch:
                             self.__nrows   = len(range(0,len(self._filelist),self.fileStep))
                         for group in result['groups']:
                             self.__peaks.append(group)
-                            self.__images[group]=Numeric.zeros((self.__nrows,self.__ncols),Numeric.Float)
-                            self.__sigmas[group]=Numeric.zeros((self.__nrows,self.__ncols),Numeric.Float)
-                        self.__images['chisq']  = Numeric.zeros((self.__nrows,self.__ncols),Numeric.Float) - 1.
+                            self.__images[group]= numpy.zeros((self.__nrows,
+                                                               self.__ncols),
+                                                               numpy.float)
+                            self.__sigmas[group]= numpy.zeros((self.__nrows,
+                                                               self.__ncols),
+                                                               numpy.float)
+                        self.__images['chisq']  = numpy.zeros((self.__nrows,
+                                                               self.__ncols),
+                                                               numpy.float) - 1.
                         if self._concentrations:
                             layerlist = concentrations['layerlist']
                             if 'mmolar' in concentrations:
@@ -645,14 +655,16 @@ class McaAdvancedFitBatch:
                             for group in concentrations['groups']:
                                 key = group+self.__conLabel
                                 self.__concentrationsKeys.append(key)
-                                self.__images[key] = Numeric.zeros((self.__nrows,self.__ncols),
-                                                                                      Numeric.Float)
+                                self.__images[key] = numpy.zeros((self.__nrows,
+                                                                  self.__ncols),
+                                                                  numpy.float)
                                 if len(layerlist) > 1:
                                     for layer in layerlist:
                                         key = group+" "+layer
                                         self.__concentrationsKeys.append(key)                                        
-                                        self.__images[key] = Numeric.zeros((self.__nrows,self.__ncols),
-                                                                                      Numeric.Float)
+                                        self.__images[key] = numpy.zeros((self.__nrows,
+                                                                    self.__ncols),
+                                                                    numpy.float)
                 for peak in self.__peaks:
                     try:
                         self.__images[peak][self.__row, self.__col] = result[peak]['fitarea']
@@ -681,7 +693,7 @@ class McaAdvancedFitBatch:
                 #this only works with EDF
                 if self.__ncols is not None:
                     if not self.counter:
-                        imgdir = os.path.join(self._outputdir,"IMAGES")
+                        imgdir = self.os_path_join(self._outputdir,"IMAGES")
                         if not os.path.exists(imgdir):
                             try:
                                 os.mkdir(imgdir)
@@ -701,8 +713,9 @@ class McaAdvancedFitBatch:
                             self.__ROIpeaks.append(group)
                             self._ROIimages[group]={}
                             for roi in dict[group].keys():
-                                self._ROIimages[group][roi]=Numeric.zeros((self.__nrows,
-                                                                            self.__ncols),Numeric.Float)
+                                self._ROIimages[group][roi]=numpy.zeros((self.__nrows,
+                                                                   self.__ncols),
+                                                                   numpy.float)
                                 
                 if not hasattr(self, "_ROIimages"):
                     print("ROI fitting only supported on EDF")
@@ -724,7 +737,7 @@ class McaAdvancedFitBatch:
         self.savedImages=[]
         if ffile is None:
             ffile = os.path.splitext(self._rootname)[0]
-            ffile = os.path.join(self.imgDir,ffile)
+            ffile = self.os_path_join(self.imgDir,ffile)
         if not self.roiFit:
             if (self.fileStep > 1) or (self.mcaStep > 1):
                 trailing = "_filestep_%02d_mcastep_%02d" % ( self.fileStep,
