@@ -1,3 +1,21 @@
+#/*##########################################################################
+# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
+#
+# This file is part of the PyMca X-ray Fluorescence Toolkit developed at
+# the ESRF by the Software group.
+#
+# This file is free software; you can redistribute it and/or modify it
+# under the terms of the GNU Lesser General Public License as published by the Free
+# Software Foundation; either version 2 of the License, or (at your option)
+# any later version.
+#
+# This file is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
+# details.
+#
+#############################################################################*/
+__author__ = "V.A. Sole - ESRF Data Analysis"
 """
 This module can be used for plugin testing purposes as well as for doing
 the bookkeeping of actual plot windows.
@@ -12,7 +30,8 @@ It implements the Plot1D interface:
     removeCurve(self, legend, replot=True)
     setActiveCurve(self)
 """
-import Plot1DBase
+from PyMca import Plot1DBase
+
 
 class Plot1D(Plot1DBase.Plot1DBase):
     def __init__(self):
@@ -20,8 +39,9 @@ class Plot1D(Plot1DBase.Plot1DBase):
         self.curveList = []
         self.curveDict = {}
         self.activeCurve = None
-        
-    def addCurve(self, x, y, legend=None, info=None, replace=False, replot=True):
+
+    def addCurve(self, x, y, legend=None, info=None, replace=False,
+                 replot=True):
         """
         Add the 1D curve given by x an y to the graph.
         """
@@ -39,7 +59,7 @@ class Plot1D(Plot1DBase.Plot1DBase):
         if replace:
             self.curveList = []
             self.curveDict = {}
-            
+
         if key in self.curveList:
             idx = self.curveList.index(key)
             self.curveList[idx] = key
@@ -56,14 +76,14 @@ class Plot1D(Plot1DBase.Plot1DBase):
         The graph will be updated if replot is true.
         """
         if legend is None:
-            return        
+            return
         if legend in self.curveList:
             idx = self.curveList.index(legend)
             del self.curveList[idx]
-        if legend in self.curveDict.keys(): 
+        if legend in self.curveDict.keys():
             del self.curveDict[legend]
         return
-    
+
     def getActiveCurve(self, just_legend=False):
         """
         Function to access the currently active curve.
@@ -139,15 +159,14 @@ class Plot1D(Plot1DBase.Plot1DBase):
 
     def getGraphXLimits(self):
         """
-        Get the graph X limits. 
+        Get the graph X limits.
         """
         xmin, ymin, xmax, ymax = self.__getAllLimits()
         return xmin, xmax
-        
 
     def getGraphYLimits(self):
         """
-        Get the graph Y (left) limits. 
+        Get the graph Y (left) limits.
         """
         xmin, ymin, xmax, ymax = self.__getAllLimits()
         return ymin, ymax
@@ -163,18 +182,21 @@ class Plot1D(Plot1DBase.Plot1DBase):
             self.activeCurve = key
         return self.activeCurve
 
-if __name__ == "__main__":
+
+def main():
     import numpy
+
     x = numpy.arange(100.)
     y = x * x
     plot = Plot1D()
     plot.addCurve(x, y, "dummy")
-    plot.addCurve(x+100, -x*x)
+    plot.addCurve(x + 100, -x * x)
     print("Active curve = ", plot.getActiveCurve())
-    print("X Limits = ",     plot.getGraphXLimits())
-    print("Y Limits = ",     plot.getGraphYLimits())
-    print("All curves = ",   plot.getAllCurves())
+    print("X Limits = ", plot.getGraphXLimits())
+    print("Y Limits = ", plot.getGraphYLimits())
+    print("All curves = ", plot.getAllCurves())
     plot.removeCurve("dummy")
-    print("All curves = ",   plot.getAllCurves())
+    print("All curves = ", plot.getAllCurves())
 
-    
+if __name__ == "__main__":
+    main()
