@@ -1,3 +1,21 @@
+#/*##########################################################################
+# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
+#
+# This file is part of the PyMca X-ray Fluorescence Toolkit developed at
+# the ESRF by the Software group.
+#
+# This file is free software; you can redistribute it and/or modify it
+# under the terms of the GNU Lesser General Public License as published by the Free
+# Software Foundation; either version 2 of the License, or (at your option)
+# any later version.
+#
+# This file is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
+# details.
+#
+#############################################################################*/
+__author__ = "V.A. Sole - ESRF Software Group"
 """
 
 Any window willing to accept 1D plugins should implement the methods defined
@@ -21,8 +39,8 @@ import sys
 import glob
 PLUGINS_DIR = None
 try:
-    if os.path.exists(os.path.join(os.path.dirname(__file__),"PyMcaPlugins")):
-        import PyMcaPlugins
+    if os.path.exists(os.path.join(os.path.dirname(__file__), "PyMcaPlugins")):
+        from PyMca import PyMcaPlugins
         PLUGINS_DIR = os.path.dirname(PyMcaPlugins.__file__)
     else:
         directory = os.path.dirname(__file__)
@@ -70,7 +88,20 @@ class Plot1DBase(object):
 
             fileList = glob.glob(os.path.join(directory, "*.py"))
             targetMethod = 'getPlugin1DInstance'
-            for module in fileList:
+            # prevent unnecessary imports
+            moduleList = []
+            for fname in fileList:
+                # in Python 3, rb implies bytes and not strings
+                f = open(fname, 'r')
+                lines = f.readlines()
+                f.close()
+                f = None
+                for line in lines:
+                    if line.startswith("def"):
+                        if line.split(" ")[1].startswith(targetMethod):
+                            moduleList.append(fname)
+                            break
+            for module in moduleList:
                 try:
                     pluginName = os.path.basename(module)[:-3]
                     if directory == PLUGINS_DIR:
@@ -114,7 +145,7 @@ class Plot1DBase(object):
         print("removeCurve not implemented")
         return None
     
-    def getActiveCurve(self):
+    def getActiveCurve(self, just_legend=False):
         """
         Function to access the currently active curve.
         It returns None in case of not having an active curve.
@@ -131,7 +162,7 @@ class Plot1DBase(object):
         print("getActiveCurve not implemented")
         return None
 
-    def getAllCurves(self):
+    def getAllCurves(self, just_legend=False):
         """
         If just_legend is False:
             It returns a list of the form:
@@ -190,6 +221,3 @@ class Plot1DBase(object):
     def getGraphYTitle(self):
         print("getGraphYTitle not implemented")
         return "Y"
-
-
-        
