@@ -3,36 +3,43 @@ __revision__ = "$Revision: 2.02 $"
 #/*##########################################################################
 # Copyright (C) 2004-2012 European Synchrotron Radiation Facility
 #
-# This file is part of the PyMCA X-ray Fluorescence Toolkit developed at
-# the ESRF by the Beamline Instrumentation Software Support (BLISS) group.
+# This file is part of the PyMca X-ray Fluorescence Toolkit developed at
+# the ESRF by the Software group.
 #
-# This toolkit is free software; you can redistribute it and/or modify it 
+# This toolkit is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the Free
-# Software Foundation; either version 2 of the License, or (at your option) 
+# Software Foundation; either version 2 of the License, or (at your option)
 # any later version.
 #
-# PyMCA is distributed in the hope that it will be useful, but WITHOUT ANY
+# PyMca is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 # FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 # details.
 #
 # You should have received a copy of the GNU General Public License along with
-# PyMCA; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
-# Suite 330, Boston, MA 02111-1307, USA.
+# PyMca; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
-# PyMCA follows the dual licensing model of Trolltech's Qt and Riverbank's PyQt
-# and cannot be used as a free plugin for a non-free program. 
+# PyMca follows the dual licensing model of Riverbank's PyQt and cannot be
+# used as a free plugin for a non-free program.
 #
-# Please contact the ESRF industrial unit (industry@esrf.fr) if this license 
+# Please contact the ESRF industrial unit (industry@esrf.fr) if this license
 # is a problem for you.
 #############################################################################*/
 import sys, getopt
 import traceback
+if sys.platform == 'win32':
+    import ctypes
+    from ctypes.wintypes import MAX_PATH
 nativeFileDialogs = None
 DEBUG = 0
 if __name__ == '__main__':
     options     = '-f'
-    longoptions = ['spec=','shm=','debug=', 'qt=', 'nativefiledialogs=']
+    longoptions = ['spec=',
+                   'shm=',
+                   'debug=',
+                   'qt=',
+                   'nativefiledialogs=']
     try:
         opts, args = getopt.getopt(
                      sys.argv[1:],
@@ -65,21 +72,22 @@ if __name__ == '__main__':
     if qtversion == '3':
         import qt
 
-import PyMcaMdi
-from PyMcaMdi import qt
+from PyMca import PyMcaMdi
+from PyMca import PyMcaQt as qt
 
 if hasattr(qt, "QString"):
     QString = qt.QString
 else:
-    QString = str
+    QString = qt.safe_str
 
 QTVERSION = qt.qVersion()
 
-from PyMca_Icons import IconDict
-from PyMca_help import HelpDict
+from PyMca.PyMca_Icons import IconDict
+from PyMca.PyMca_help import HelpDict
+from PyMca import PyMcaDataDir
 import os
-__version__ = "4.5.0"
-if (QTVERSION < '4.0.0') and ((sys.platform == 'darwin') or (QTVERSION < '3.0.0')):
+__version__ = "4.6.2"
+if (QTVERSION < '4.0.0') and (sys.platform == 'darwin'):
     class SplashScreen(qt.QWidget):
         def __init__(self,parent=None,name="SplashScreen",
                         fl=qt.Qt.WStyle_Customize  | qt.Qt.WDestructiveClose,
@@ -104,15 +112,15 @@ if (QTVERSION < '4.0.0') and ((sys.platform == 'darwin') or (QTVERSION < '3.0.0'
 
 if __name__ == "__main__":
     app = qt.QApplication(sys.argv)
-    if QTVERSION >= '3.0.0':
-        strlist = qt.QStyleFactory.keys()
-        if sys.platform == "win32":
-            for item in strlist:
-                text = str(item)
-                if text == "WindowsXP":
-                    style = qt.QStyleFactory.create(item)
-                    app.setStyle(style)
-                    break
+    strlist = qt.QStyleFactory.keys()
+    if sys.platform == "win32":
+        for item in strlist:
+            text = str(item)
+            if text == "WindowsXP":
+                style = qt.QStyleFactory.create(item)
+                app.setStyle(style)
+                break
+    # TODO why this strange test
     if 1 or QTVERSION < '4.0.0':
         winpalette = qt.QPalette(qt.QColor(230,240,249),qt.QColor(238,234,238))
         app.setPalette(winpalette)
@@ -122,12 +130,12 @@ if __name__ == "__main__":
         palette.setColor(role, qt.QColor(238,234,238))
         app.setPalette(palette)
 
-    mpath = os.path.dirname(PyMcaMdi.__file__)
+    mpath = PyMcaDataDir.PYMCA_DATA_DIR
     if mpath[-3:] == "exe":
         mpath = os.path.dirname(mpath)
     if QTVERSION < '4.0.0':
         qt.QMimeSourceFactory.defaultFactory().addFilePath(mpath)
-        if (sys.platform == 'darwin') or (qt.qVersion() < '3.0.0'):
+        if (sys.platform == 'darwin'):
             pixmap = qt.QPixmap('PyMcaSplashImage.png')    
             splash  = SplashScreen(pixmap=pixmap)
             splash.message( 'PyMCA version %s\n' % __version__)     
@@ -156,7 +164,7 @@ if __name__ == "__main__":
         else:
             splash = qt.QSplashScreen()
         splash.show()
-        import ChangeLog
+        from PyMca import ChangeLog
         font = splash.font()
         font.setBold(1)
         splash.setFont(font)
@@ -166,47 +174,47 @@ if __name__ == "__main__":
         if sys.platform == "darwin":
             qt.qApp.processEvents()
 
-import McaWindow
-import ScanWindow
+from PyMca import McaWindow
+from PyMca import ScanWindow
 OBJECT3D = False
 if QTVERSION > '4.0.0':
-    import PyMcaImageWindow
-    import PyMcaHKLImageWindow
+    from PyMca import PyMcaImageWindow
+    from PyMca import PyMcaHKLImageWindow
     try:
         #This is to make sure it is properly frozen
         #and that Object3D is fully supported
         import OpenGL.GL
         import PyQt4.QtOpenGL
         #import Object3D.SceneGLWindow as SceneGLWindow
-        import PyMcaGLWindow as SceneGLWindow
+        import PyMca.PyMcaGLWindow as SceneGLWindow
         OBJECT3D = True
     except:
         OBJECT3D = False
-import QDispatcher
-import ElementsInfo
-import PeakIdentifier
-import PyMcaBatch
+from PyMca import QDispatcher
+from PyMca import ElementsInfo
+from PyMca import PeakIdentifier
+from PyMca import PyMcaBatch
 ###########import Fit2Spec
-import Mca2Edf
+from PyMca import Mca2Edf
 STACK = False
 if QTVERSION > '4.0.0':
     try:
-        import QStackWidget
-        import StackSelector
+        from PyMca import QStackWidget
+        from PyMca import StackSelector
         STACK = True
     except:
         STACK = False
-    import PyMcaPostBatch
-    import RGBCorrelator
-    import MaterialEditor
+    from PyMca import PyMcaPostBatch
+    from PyMca import RGBCorrelator
+    from PyMca import MaterialEditor
 
-import ConfigDict
-import PyMcaDirs
+from PyMca import ConfigDict
+from PyMca import PyMcaDirs
 
 XIA_CORRECT = False
 if QTVERSION > '4.3.0':
     try:
-        import XiaCorrect
+        from PyMca import XiaCorrect
         XIA_CORRECT = True
     except:
         pass
@@ -226,17 +234,17 @@ if (sys.platform != 'win32') and (sys.platform != 'darwin'):
     SOURCESLIST.append("SPS")
 """
 
-class PyMca(PyMcaMdi.PyMca):
+class PyMcaMain(PyMcaMdi.PyMcaMdi):
     def __init__(self, parent=None, name="PyMca", fl=None,**kw):
             if QTVERSION < '4.0.0':
                 if fl is None:qt.Qt.WDestructiveClose
-                PyMcaMdi.PyMca.__init__(self, parent, name, fl)
+                PyMcaMdi.PyMcaMdi.__init__(self, parent, name, fl)
                 self.setCaption(name)
                 self.setIcon(qt.QPixmap(IconDict['gioconda16']))
                 self.menuBar().setIcon(qt.QPixmap(IconDict['gioconda16']))            
             else:
                 if fl is None: fl = qt.Qt.WA_DeleteOnClose
-                PyMcaMdi.PyMca.__init__(self, parent, name, fl)
+                PyMcaMdi.PyMcaMdi.__init__(self, parent, name, fl)
                 maxheight = qt.QDesktopWidget().height()
                 if maxheight < 799:
                     self.setMinimumHeight(int(0.8*maxheight))
@@ -261,12 +269,16 @@ class PyMca(PyMcaMdi.PyMca):
                 self.openMenu = qt.QPopupMenu()
                 self.openMenu.insertItem("PyMca Configuration",0)
                 self.openMenu.insertItem("Data Source",1)
-                self.connect(self.openMenu,qt.SIGNAL('activated(int)'),self.openSource)
+                self.connect(self.openMenu,qt.SIGNAL('activated(int)'),
+                             self.openSource)
             else:
                 self.openMenu = qt.QMenu()
                 self.openMenu.addAction("PyMca Configuration", self.openSource)
                 self.openMenu.addAction("Data Source",
                              self.sourceWidget.sourceSelector._openFileSlot)
+                self.openMenu.addAction("Load Training Data",
+                                            self.loadTrainingData)
+
                 #self.connect(self.openMenu,qt.SIGNAL('activated(int)'),self.openSource)
 
             if QTVERSION > '4.0.0':
@@ -374,8 +386,12 @@ class PyMca(PyMcaMdi.PyMca):
                 else:
                     self.mcawindow.showMaximized()
             currentConfigDict = ConfigDict.ConfigDict()
-            defaultFileName = self.__getDefaultSettingsFile()
-            self.configDir  = os.path.dirname(defaultFileName)
+            try:
+                defaultFileName = self.__getDefaultSettingsFile()
+                self.configDir  = os.path.dirname(defaultFileName)
+            except:
+                if not ('fresh' in kw):
+                    raise
             if not ('fresh' in kw):
                 if os.path.exists(defaultFileName):
                     currentConfigDict.read(defaultFileName)
@@ -1284,32 +1300,46 @@ class PyMca(PyMcaMdi.PyMca):
         else:self.identifier.raise_()
             
     def __batchFitting(self):
-        if self.__batch is None:self.__batch = PyMcaBatch.McaBatchGUI(fl=0,actions=1)
-        if self.__batch.isHidden():self.__batch.show()
-        if QTVERSION < '4.0.0': self.__batch.raiseW()
-        else: self.__batch.raise_()
+        if self.__batch is None:
+            self.__batch = PyMcaBatch.McaBatchGUI(fl=0,actions=1)
+        if self.__batch.isHidden():
+            self.__batch.show()
+        if QTVERSION < '4.0.0':
+            self.__batch.raiseW()
+        else:
+            self.__batch.raise_()
 
     def __mca2EdfConversion(self):
-        if self.__mca2Edf is None:self.__mca2Edf = Mca2Edf.Mca2EdfGUI(fl=0,actions=1)
-        if self.__mca2Edf.isHidden():self.__mca2Edf.show()
-        if QTVERSION < '4.0.0': self.__mca2Edf.raiseW()
-        else:self.__mca2Edf.raise_()
+        if self.__mca2Edf is None:
+            self.__mca2Edf = Mca2Edf.Mca2EdfGUI(fl=0,actions=1)
+        if self.__mca2Edf.isHidden():
+            self.__mca2Edf.show()
+        if QTVERSION < '4.0.0':
+            self.__mca2Edf.raiseW()
+        else:
+            self.__mca2Edf.raise_()
 
     def __fit2SpecConversion(self):
-        if self.__fit2Spec is None:self.__fit2Spec = Fit2Spec.Fit2SpecGUI(fl=0,actions=1)
-        if self.__fit2Spec.isHidden():self.__fit2Spec.show()
-        if QTVERSION < '4.0.0': self.__fit2Spec.raiseW()
-        else:self.__fit2Spec.raise_()
+        if self.__fit2Spec is None:
+            self.__fit2Spec = Fit2Spec.Fit2SpecGUI(fl=0,actions=1)
+        if self.__fit2Spec.isHidden():
+            self.__fit2Spec.show()
+        if QTVERSION < '4.0.0':
+            self.__fit2Spec.raiseW()
+        else:
+            self.__fit2Spec.raise_()
 
     def __rgbCorrelator(self):
-        if self.__correlator is None:self.__correlator = []
+        if self.__correlator is None:
+            self.__correlator = []
         fileTypeList = ["Batch Result Files (*dat)",
                         "EDF Files (*edf)",
                         "EDF Files (*ccd)",
                         "All Files (*)"]
         message = "Open ONE Batch result .dat file or SEVERAL EDF files"
         filelist = self.__getStackOfFiles(fileTypeList, message) 
-        if not(len(filelist)): return
+        if not(len(filelist)):
+            return
         filelist.sort()
         self.sourceWidget.sourceSelector.lastInputDir = os.path.dirname(filelist[0])
         PyMcaDirs.inputDir = os.path.dirname(filelist[0])
@@ -1336,7 +1366,7 @@ class PyMca(PyMcaMdi.PyMca):
                     del self.__correlator[i]
                     break
 
-    def __getStackOfFiles(self, typelist, message = "", getfilter=False):
+    def __getStackOfFiles(self, typelist, message="", getfilter=False):
         wdir = PyMcaDirs.inputDir
         fileTypeList = typelist
         filterused = None
@@ -1370,7 +1400,7 @@ class PyMca(PyMcaMdi.PyMca):
                     else:
                         return []
                 else:
-                    sample  = str(filelist[0])
+                    sample  = qt.safe_str(filelist[0])
                     for filetype in fileTypeList:
                         ftype = filetype.replace("(", "").replace(")","")
                         extensions = ftype.split()[2:]
@@ -1399,7 +1429,7 @@ class PyMca(PyMcaMdi.PyMca):
                 if ret == qt.QDialog.Accepted:
                     filelist = fdialog.selectedFiles()
                     if getfilter:
-                        filterused = str(fdialog.selectedFilter())
+                        filterused = qt.safe_str(fdialog.selectedFilter())
                     fdialog.close()
                     del fdialog                        
                 else:
@@ -1409,7 +1439,7 @@ class PyMca(PyMcaMdi.PyMca):
                         return [], filterused
                     else:
                         return []
-        filelist = [str(x) for x in filelist]
+        filelist = [qt.safe_str(x) for x in filelist]
         if getfilter:
             return filelist, filterused
         else:
@@ -1520,12 +1550,12 @@ class PyMca(PyMcaMdi.PyMca):
         if QTVERSION < '4.0.0':ret = outfile.exec_loop()
         else:ret = outfile.exec_()
         if ret:
-            filterused = str(outfile.selectedFilter()).split()
+            filterused = qt.safe_str(outfile.selectedFilter()).split()
             extension = ".ini"
             if QTVERSION < '4.0.0':
-                outdir=str(outfile.selectedFile())
+                outdir=qt.safe_str(outfile.selectedFile())
             else:
-                outdir=str(outfile.selectedFiles()[0])
+                outdir=qt.safe_str(outfile.selectedFiles()[0])
             try:
                 outputDir  = os.path.dirname(outdir)
             except:
@@ -1578,20 +1608,27 @@ class PyMca(PyMcaMdi.PyMca):
     def __getDefaultSettingsFile(self):
         filename = "PyMca.ini"
         if sys.platform == 'win32':
-            home = os.getenv('USERPROFILE')
-            try:
-                l = len(home)
-                directory = os.path.join(home,"My Documents")
-            except:
-                home = '\\'
-                directory = '\\'
-            #print home
-            #print directory
+            # recipe based on: http://bugs.python.org/issue1763#msg62242
+            dll = ctypes.windll.shell32
+            buf = ctypes.create_unicode_buffer(MAX_PATH + 1)
+            if dll.SHGetSpecialFolderPathW(None, buf, 0x0005, False):
+                directory = buf.value
+            else:
+                # the above should have worked
+                home = os.getenv('USERPROFILE')
+                try:
+                    l = len(home)
+                    directory = os.path.join(home, "My Documents")
+                except:
+                    home = '\\'
+                    directory = '\\'
+                #print home
+                #print directory
             if os.path.isdir('%s' % directory):
-                directory = os.path.join(directory,"PyMca")
+                directory = os.path.join(directory, "PyMca")
             else:
                 #print "My Documents is not there"
-                directory = os.path.join(home,"PyMca")
+                directory = os.path.join(home, "PyMca")
             if not os.path.exists('%s' % directory):
                 #print "PyMca directory not present"
                 os.mkdir('%s' % directory)
@@ -1600,12 +1637,26 @@ class PyMca(PyMcaMdi.PyMca):
             #print finalfile
         else:
             home = os.getenv('HOME')
-            directory = os.path.join(home,"PyMca")
+            directory = os.path.join(home, "PyMca")
             if not os.path.exists('%s' % directory):
                 os.mkdir('%s' % directory)
             finalfile =  os.path.join(directory, filename)
         return finalfile
 
+    def loadTrainingData(self):
+        try:
+            source = os.path.join(PyMcaDataDir.PYMCA_DATA_DIR,
+                                    'XRFSpectrum.mca')
+            self.sourceWidget.sourceSelector.openSource(source)
+        except:
+            msg = qt.QMessageBox(self)
+            msg.setIcon(qt.QMessageBox.Critical)
+            msg.setWindowTitle("Error opening data source")
+            msg.setText("Cannot open data source %s" % source)
+            msg.setInformativeText(str(sys.exc_info()[1]))
+            msg.setDetailedText(traceback.format_exc())
+            msg.exec_()
+
     def openSource(self,index=0):
         if DEBUG:
             print("index = %d " % index)
@@ -1626,8 +1677,10 @@ class PyMca(PyMcaMdi.PyMca):
                 outfile.setFileMode(outfile.ExistingFile)
                 ret = outfile.exec_()
             if ret:
-                if QTVERSION < '4.0.0':filename=str(outfile.selectedFile())
-                else:filename=str(outfile.selectedFiles()[0])
+                if QTVERSION < '4.0.0':
+                    filename = qt.safe_str(outfile.selectedFile())
+                else:
+                    filename = qt.safe_str(outfile.selectedFiles()[0])
                 outfile.close()
                 del outfile
             else:
@@ -1657,19 +1710,22 @@ class PyMca(PyMcaMdi.PyMca):
                 self.menuBrowser.setCaption(QString("Main Menu Help"))
             else:
                 self.menuBrowser.setWindowTitle(QString("Main Menu Help"))
-            dir=os.path.dirname(PyMcaMdi.__file__)
-            if not os.path.exists(os.path.join(dir,"HTML","Menu.html")):
-                dir = os.path.dirname(dir)
+            ddir=PyMcaDataDir.PYMCA_DOC_DIR
+            if not os.path.exists(os.path.join(ddir,"HTML","Menu.html")):
+                ddir = os.path.dirname(ddir)
             if QTVERSION < '4.0.0':
-                self.menuBrowser.mimeSourceFactory().addFilePath(QString(dir+"/HTML"))
+                self.menuBrowser.mimeSourceFactory().addFilePath(QString(ddir+"/HTML"))
                 self.menuBrowser.setSource(QString("Menu.html"))
             else:
-                self.menuBrowser.setSearchPaths([os.path.join(dir,"HTML")])
+                self.menuBrowser.setSearchPaths([os.path.join(ddir,"HTML")])
                 self.menuBrowser.setSource(qt.QUrl(QString("Menu.html")))
             self.menuBrowser.show()
-        if self.menuBrowser.isHidden():self.menuBrowser.show()
-        if QTVERSION < '4.0.0': self.menuBrowser.raiseW()
-        else: self.menuBrowser.raise_()
+        if self.menuBrowser.isHidden():
+            self.menuBrowser.show()
+        if QTVERSION < '4.0.0':
+            self.menuBrowser.raiseW()
+        else:
+            self.menuBrowser.raise_()
 
     def onDisplayHowto(self):
         if self.displayBrowser is None:
@@ -1678,19 +1734,22 @@ class PyMca(PyMcaMdi.PyMca):
                 self.displayBrowser.setCaption(QString("Data Display HOWTO"))
             else:
                 self.displayBrowser.setWindowTitle(QString("Data Display HOWTO"))                
-            dir=os.path.dirname(PyMcaMdi.__file__)
-            if not os.path.exists(os.path.join(dir,"HTML","Display-HOWTO.html")):
-                dir = os.path.dirname(dir)
+            ddir=PyMcaDataDir.PYMCA_DOC_DIR
+            if not os.path.exists(os.path.join(ddir,"HTML","Display-HOWTO.html")):
+                ddir = os.path.dirname(ddir)
             if QTVERSION < '4.0.0':
-                self.displayBrowser.mimeSourceFactory().addFilePath(QString(dir+"/HTML"))
+                self.displayBrowser.mimeSourceFactory().addFilePath(QString(ddir+"/HTML"))
                 self.displayBrowser.setSource(QString("Display-HOWTO.html"))
             else:
-                self.displayBrowser.setSearchPaths([os.path.join(dir,"HTML")])
+                self.displayBrowser.setSearchPaths([os.path.join(ddir,"HTML")])
                 self.displayBrowser.setSource(qt.QUrl(QString("Display-HOWTO.html")))
             self.displayBrowser.show()
-        if self.displayBrowser.isHidden():self.displayBrowser.show()
-        if QTVERSION < '4.0.0':self.displayBrowser.raiseW()
-        else:self.displayBrowser.raise_()
+        if self.displayBrowser.isHidden():
+            self.displayBrowser.show()
+        if QTVERSION < '4.0.0':
+            self.displayBrowser.raiseW()
+        else:
+            self.displayBrowser.raise_()
     
     def onMcaHowto(self):
         if self.mcaBrowser is None:
@@ -1699,14 +1758,14 @@ class PyMca(PyMcaMdi.PyMca):
                 self.mcaBrowser.setCaption(QString("MCA HOWTO"))
             else:
                 self.mcaBrowser.setWindowTitle(QString("MCA HOWTO"))
-            dir=os.path.dirname(PyMcaMdi.__file__)
-            if not os.path.exists(dir+"/HTML"+"/MCA-HOWTO.html"):
-                dir = os.path.dirname(dir)
+            ddir=PyMcaDataDir.PYMCA_DOC_DIR
+            if not os.path.exists(ddir+"/HTML"+"/MCA-HOWTO.html"):
+                ddir = os.path.dirname(ddir)
             if QTVERSION < '4.0.0':
-                self.mcaBrowser.mimeSourceFactory().addFilePath(QString(dir+"/HTML"))
+                self.mcaBrowser.mimeSourceFactory().addFilePath(QString(ddir+"/HTML"))
                 self.mcaBrowser.setSource(QString("MCA-HOWTO.html"))
             else:
-                self.mcaBrowser.setSearchPaths([os.path.join(dir,"HTML")])
+                self.mcaBrowser.setSearchPaths([os.path.join(ddir,"HTML")])
                 self.mcaBrowser.setSource(qt.QUrl(QString("MCA-HOWTO.html")))
                 #f = open(os.path.join(dir,"HTML","MCA-HOWTO.html"))
                 #self.mcaBrowser.setHtml(f.read())
@@ -1727,7 +1786,8 @@ class PyMca(PyMcaMdi.PyMca):
             self.changeLog.setCursor(self.cursor())
             self.changeLog.setWindowTitle("PyMCA %s Changes" % __version__)
             self.changeLog.setWindowIcon(qt.QIcon(qt.QPixmap(IconDict['gioconda16'])))
-            mpath = os.path.dirname(PyMcaMdi.__file__)
+            # Does this belong to the data dir or the doc dir?
+            mpath = PyMcaDataDir.PYMCA_DATA_DIR
             fname = os.path.join(mpath,'changelog.txt')
             if not os.path.exists(fname):
                while len(mpath) > 3:
@@ -1768,8 +1828,8 @@ class PyMca(PyMcaMdi.PyMca):
             else:self.mcawindow.raise_()
             self.mcawindow.graph.printps()    
 
-    def __McaWindowSignal(self,dict):
-        if dict['event'] == 'NewScanCurve':
+    def __McaWindowSignal(self, ddict):
+        if ddict['event'] == 'NewScanCurve':
             if self.mcawindow.scanwindow.isHidden():
                 self.mcawindow.scanwindow.show()
             self.mcawindow.scanwindow.setFocus()   
@@ -1812,8 +1872,8 @@ if 0:
             Or None if not found
             """
             for graph in self.mdi.windowList():
-                    if str(graph.caption())==name:
-                            return graph
+                if qt.safe_str(graph.caption())== name:
+                    return graph
             return None
 
     def closeGraph(self, name):
@@ -1837,10 +1897,10 @@ class MyQTextBrowser(qt.QTextBrowser):
     def  setSource(self,name):
         if name == QString("./PyMCA.html"):
             if sys.platform == 'win32':
-                dir=os.path.dirname(PyMcaMdi.__file__)
-                if not os.path.exists(dir+"/HTML"+"/PyMCA.html"):
-                    dir = os.path.dirname(dir)
-                cmd = dir+"/HTML/PyMCA.pdf"
+                ddir=PyMcaDataDir.PYMCA_DOC_DIR
+                if not os.path.exists(ddir+"/HTML"+"/PyMCA.html"):
+                    ddir = os.path.dirname(ddir)
+                cmd = ddir+"/HTML/PyMCA.pdf"
                 os.system('"%s"' % cmd)
                 return
             try:
@@ -1848,8 +1908,9 @@ class MyQTextBrowser(qt.QTextBrowser):
             except:
                 self.report = qt.QTextBrowser()
                 self.report.setCaption(QString("PyMca Report"))
-                self.report.mimeSourceFactory().addFilePath(QString(os.path.dirname(PyMcaMdi.__file__)+"/HTML"))
-                self.report.mimeSourceFactory().addFilePath(QString(os.path.dirname(PyMcaMdi.__file__)+"/HTML/PyMCA_files"))
+                ddir=PyMcaDataDir.PYMCA_DOC_DIR
+                self.report.mimeSourceFactory().addFilePath(QString(ddir+"/HTML"))
+                self.report.mimeSourceFactory().addFilePath(QString(ddir+"/HTML/PyMCA_files"))
                 self.report.setSource(name)
             if self.report.isHidden():self.report.show()
             self.report.raiseW()
@@ -1885,7 +1946,7 @@ if __name__ == '__main__':
     if PROFILING:
         import profile
         import pstats
-    PyMcaMainWidgetInstance = PyMca(**keywords)
+    PyMcaMainWidgetInstance = PyMcaMain(**keywords)
     if nativeFileDialogs is not None:
         PyMcaDirs.nativeFileDialogs = nativeFileDialogs
     if debugreport:PyMcaMainWidgetInstance.onDebug()
