@@ -1,9 +1,37 @@
+#/*##########################################################################
+# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
+#
+# This file is part of the PyMca X-ray Fluorescence Toolkit developed at
+# the ESRF by the Software group.
+#
+# This toolkit is free software; you can redistribute it and/or modify it
+# under the terms of the GNU General Public License as published by the Free
+# Software Foundation; either version 2 of the License, or (at your option)
+# any later version.
+#
+# PyMca is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# details.
+#
+# You should have received a copy of the GNU General Public License along with
+# PyMca; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# PyMca follows the dual licensing model of Riverbank's PyQt and cannot be
+# used as a free plugin for a non-free program.
+#
+# Please contact the ESRF industrial unit (industry@esrf.fr) if this license
+# is a problem for you.
+#############################################################################*/
+__author__ = "V.A. Sole - ESRF Data Analysis"
 """
 
-A Stack plugin is a module that will be automatically added to the PyMca stack windows
-in order to perform user defined operations on the data stack.
+A Stack plugin is a module that will be automatically added to the PyMca
+stack windows in order to perform user defined operations on the data stack.
 
-These plugins will be compatible with any stack window that provides the functions:
+These plugins will be compatible with any stack window that provides the
+functions:
     #data related
     getStackDataObject
     getStackData
@@ -29,60 +57,53 @@ These plugins will be compatible with any stack window that provides the functio
     selectionMaskUpdated
 """
 try:
-    import StackPluginBase
-except ImportError:
-    from . import StackPluginBase
-try:
+    from PyMca import StackPluginBase
     from PyMca import SGWindow
     from PyMca import SNIPWindow
     import PyMca.PyMca_Icons as PyMca_Icons
 except ImportError:
     print("Plugin importing from somewhere else")
+    import StackPluginBase
     import SGWindow
     import SNIPWindow
     import PyMca_Icons
 
 DEBUG = 0
 
+
 class BackgroundStackPlugin(StackPluginBase.StackPluginBase):
     def __init__(self, stackWindow, **kw):
         StackPluginBase.DEBUG = DEBUG
         StackPluginBase.StackPluginBase.__init__(self, stackWindow, **kw)
-        SGtext  = "Replace current stack by a\n"
+        SGtext = "Replace current stack by a\n"
         SGtext += "Savitsky-Golay treated one."
-        SNIP1Dtext  = "Replace current stack by a\n"
+        SNIP1Dtext = "Replace current stack by a\n"
         SNIP1Dtext += "SNIP1D treated one."
-        SNIP2Dtext  = "Replace current stack by a\n"
+        SNIP2Dtext = "Replace current stack by a\n"
         SNIP2Dtext += "SNIP2D treated one."
         self.methodDict = {}
         function = self.replaceStackWithSavitzkyGolayFiltering
         info = SGtext
         icon = PyMca_Icons.substract
-        self.methodDict["Savitzky-Golay Filtering"] =[function,
-                                                      info,
-                                                      icon]
+        self.methodDict["Savitzky-Golay Filtering"] = [function, info, icon]
         function = self.subtract1DSnipBackground
         info = SNIP1Dtext
-        self.methodDict["Subtract SNIP 1D Background"] =[function,
-                                                      info,
-                                                      icon]
+        self.methodDict["Subtract SNIP 1D Background"] = [function, info, icon]
         function = self.replaceWith1DSnipBackground
-        info  = "Smooth and replace current stack\n"
+        info = "Smooth and replace current stack\n"
         info += "by its SNIP1D background."
-        self.methodDict["Deglitch with SNIP 1D Background"] =[function,
+        self.methodDict["Deglitch with SNIP 1D Background"] = [function,
                                                               info,
                                                               PyMca_Icons.smooth]
         function = self.subtract2DSnipBackground
         info = SNIP2Dtext
-        self.methodDict["Subtract SNIP 2D Background"] =[function,
-                                                      info,
-                                                      icon]
+        self.methodDict["Subtract SNIP 2D Background"] = [function, info, icon]
 
         self.__methodKeys = ["Savitzky-Golay Filtering",
                              "Deglitch with SNIP 1D Background",
                              "Subtract SNIP 1D Background",
                              "Subtract SNIP 2D Background"]
-                                     
+
     def stackUpdated(self):
         self.dialogWidget = None
 
@@ -152,7 +173,7 @@ class BackgroundStackPlugin(StackPluginBase.StackPluginBase):
         if not len(imageList):
             return
         snipWindow = SNIPWindow.SNIPDialog(None,
-                                           imageList[0]*1)
+                                           imageList[0] * 1)
         #snipWindow.setModal(True)
         snipWindow.show()
         ret = snipWindow.exec_()
@@ -165,8 +186,9 @@ class BackgroundStackPlugin(StackPluginBase.StackPluginBase):
             function(stack, *arguments)
             self.setStack(stack)
 
-
 MENU_TEXT = "Stack Filtering Options"
+
+
 def getStackPluginInstance(stackWindow, **kw):
     ob = BackgroundStackPlugin(stackWindow)
     return ob
