@@ -1,27 +1,27 @@
 #/*##########################################################################
-# Copyright (C) 2004-2011 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
 #
-# This file is part of the PyMCA X-ray Fluorescence Toolkit developed at
-# the ESRF by the Beamline Instrumentation Software Support (BLISS) group.
+# This file is part of the PyMca X-ray Fluorescence Toolkit developed at
+# the ESRF by the Software group.
 #
-# This toolkit is free software; you can redistribute it and/or modify it 
+# This toolkit is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the Free
-# Software Foundation; either version 2 of the License, or (at your option) 
+# Software Foundation; either version 2 of the License, or (at your option)
 # any later version.
 #
-# PyMCA is distributed in the hope that it will be useful, but WITHOUT ANY
+# PyMca is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 # FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 # details.
 #
 # You should have received a copy of the GNU General Public License along with
-# PyMCA; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
-# Suite 330, Boston, MA 02111-1307, USA.
+# PyMca; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
-# PyMCA follows the dual licensing model of Trolltech's Qt and Riverbank's PyQt
-# and cannot be used as a free plugin for a non-free program. 
+# PyMca follows the dual licensing model of Riverbank's PyQt and cannot be
+# used as a free plugin for a non-free program.
 #
-# Please contact the ESRF industrial unit (industry@esrf.fr) if this license 
+# Please contact the ESRF industrial unit (industry@esrf.fr) if this license
 # is a problem for you.
 #############################################################################*/
 """
@@ -29,21 +29,21 @@ Demo example of generic access to data sources.
 
 """
 import sys
-import PyMcaQt as qt
+import os
+from PyMca import PyMcaQt as qt
 QTVERSION = qt.qVersion()
 
 #import QSPSDataSource
 #import QSpecFileDataSource
 #import QEdfFileDataSource
-import SpecFileDataSource
-import EdfFileDataSource
-import QEdfFileWidget
-import os
+from PyMca import SpecFileDataSource
+from PyMca import EdfFileDataSource
+from PyMca import QEdfFileWidget
 if 0 and QTVERSION < '4.0.0':
-    import MySpecFileSelector as QSpecFileWidget
+    from PyMca import MySpecFileSelector as QSpecFileWidget
     QSpecFileWidget.QSpecFileWidget = QSpecFileWidget.SpecFileSelector
 else:
-    import QSpecFileWidget
+    from PyMca import QSpecFileWidget
 
 if (sys.platform == "win32") or (sys.platform == "darwin"):
     source_types = { SpecFileDataSource.SOURCE_TYPE: SpecFileDataSource.SpecFileDataSource,
@@ -54,9 +54,9 @@ if (sys.platform == "win32") or (sys.platform == "darwin"):
     sps = None 
 else:
     #import SpsDataSource
-    import QSpsDataSource
+    from PyMca import QSpsDataSource
     sps = QSpsDataSource.SpsDataSource.sps
-    import QSpsWidget
+    from PyMca import QSpsWidget
     source_types = { SpecFileDataSource.SOURCE_TYPE: SpecFileDataSource.SpecFileDataSource,
                      EdfFileDataSource.SOURCE_TYPE:  EdfFileDataSource.EdfFileDataSource,
                      QSpsDataSource.SOURCE_TYPE: QSpsDataSource.QSpsDataSource}
@@ -67,8 +67,8 @@ else:
 
 NEXUS = True
 try:
-    import NexusDataSource
-    import PyMcaNexusWidget
+    from PyMca import NexusDataSource
+    from PyMca import PyMcaNexusWidget
     import h5py
 except:
     NEXUS = False
@@ -136,9 +136,25 @@ def getSourceType(sourceName0):
             mccd = False
         if (line.startswith("{")) or mccd:
             return EdfFileDataSource.SOURCE_TYPE
+        elif sourceName.lower().endswith('edf.gz') or\
+             sourceName.lower().endswith('ccd.gz') or\
+             sourceName.lower().endswith('raw.gz') or\
+             sourceName.lower().endswith('edf.bz2') or\
+             sourceName.lower().endswith('ccd.bz2') or\
+             sourceName.lower().endswith('raw.bz2'):
+            return EdfFileDataSource.SOURCE_TYPE
         else:
             if NEXUS:
-                if h5py.is_hdf5(sourceName):
+                ishdf5 = False
+                try:
+                    ishdf5 = h5py.is_hdf5(sourceName)
+                except TypeError:
+                    if sys.version > '2.9':
+                        if sourceName.endswith('.h5') or\
+                           sourceName.endswith('.hdf') or\
+                           sourceName.endswith('.nxs'):
+                            ishdf5 = True
+                if ishdf5:
                     return NexusDataSource.SOURCE_TYPE
                 try:
                     f = h5py.File(sourceName, 'r')
@@ -165,7 +181,6 @@ def QDataSource(name=None, source_type=None):
   
   
 if __name__ == "__main__":
-    import sys,time
     try:
         sourcename=sys.argv[1]
         key       =sys.argv[2]        
